/**
 * 
 */
package org.enhydra.dm.business;

import java.io.File;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Properties;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.DocumentManager;
import org.enhydra.dm.api.DocumentVersion;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.business.exceptions.ActionNotAllowedException;
import org.enhydra.dm.business.exceptions.DatabaseException;
import org.enhydra.dm.business.exceptions.DocumentStoreException;
import org.enhydra.dm.business.exceptions.LockException;
import org.enhydra.dm.data.DocumentDO;
import org.enhydra.dm.data.DocumentQuery;
import org.enhydra.dm.data.DocumentVersionDO;
import org.enhydra.dm.data.DocumentVersionQuery;
import org.enhydra.dm.data.FoDocumentDO;

import com.lutris.appserver.server.sql.DBRowUpdateException;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.dods.builder.generator.query.NonUniqueQueryException;
import com.lutris.dods.builder.generator.query.QueryBuilder;
import com.lutris.dods.builder.generator.query.QueryException;
import com.lutris.dods.builder.generator.query.RefAssertionException;

/**
 * Default implementation of DocumentManager interface.
 * 
 * @author Slobodan Vujasinovic
 */
public class DocumentManagerImpl implements DocumentManager {

   private Log logger = null;

   /**
    * Default constructor
    */

   public DocumentManagerImpl() {
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#checkOut(java.lang.String, java.lang.String)
    */

   public Document checkOut(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);

         if (documentDO.getLOCKEDBY() != null || documentDO.getCHECKEDOUTBY() != null) {
            throw new LockException("Document can't be checked out (id)! " + documentId);
         }

         documentDO.setLOCKEDBY(userName);
         documentDO.setCHECKEDOUTBY(userName);
         documentDO.save();
         return new DocumentImpl(documentDO);
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#unCheckOut(java.lang.String,
    *      java.lang.String)
    */
   public void unCheckOut(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);

         String coUserName = documentDO.getCHECKEDOUTBY();
         if (coUserName == null) {
            throw new LockException("Document isn't checked out (id)! " + documentId);
         }

         if (!coUserName.equals(userName)) {
            throw new LockException("User can't un check out document (id)! "
                                    + documentId);
         }

         documentDO.setLOCKEDBY(null);
         documentDO.setCHECKEDOUTBY(null);
         documentDO.save();

         if (null != logger) {
            logger.log(Log.DEBUG, "Document is Un Checked Out (id):"
                                  + documentId + " by user(id):" + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#checkIn(java.lang.String, java.lang.String,
    *      java.lang.String)
    */
   public void checkIn(String documentId, String documentPath, String userName)
      throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);
         String coUserName = documentDO.getCHECKEDOUTBY();
         if (coUserName == null) {
            throw new LockException("Document isn't checked out (id)! " + documentId);
         }

         if (!coUserName.equals(userName)) {
            throw new LockException("User can't check in document (id)! " + documentId);
         }

         if (null == documentPath) {
            throw new DocumentStoreException("Error while saving document content for document (id): "
                                             + documentId);
         }

         File documentFile = new File(documentPath);
         if (documentFile == null || !documentFile.exists()) {
            throw new DocumentStoreException("Error while saving document content for document (id): "
                                             + documentId);
         }

         if (documentDO.getAUTOVERSIONABLE()) {
            DocumentVersionDO dvDO = createDocumentVersion(documentDO, userName);
            if (null == dvDO) {
               throw new DatabaseException("Document Version creation error!");
            }
            dvDO.save();
         }

         documentDO.setFILEPATH(documentPath);
         documentDO.setSIZE(new BigDecimal(documentFile.length()));
         documentDO.setLASTMODIFIEDDATE(new Timestamp(System.currentTimeMillis()));
         documentDO.setLASTMODIFIEDBY(userName);
         documentDO.setLOCKEDBY(null);
         documentDO.setCHECKEDOUTBY(null);
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document is Checked In (id):"
                                  + documentId + " by user(id):" + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#unlock(java.lang.String, java.lang.String)
    */
   public void unlock(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);

         String lockUserName = documentDO.getLOCKEDBY();
         if (lockUserName == null) {
            throw new LockException("Document isn't locked (id)! " + documentId);
         }

         if (!lockUserName.equals(userName)) {
            throw new LockException("User can't unlock document (id)! " + documentId);
         }

         documentDO.setLOCKEDBY(null);
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document is Un Locked (id):"
                                  + documentId + " by user(id):" + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#lock(java.lang.String, java.lang.String)
    */
   public void lock(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);

         if (documentDO.getLOCKEDBY() != null || documentDO.getCHECKEDOUTBY() != null) {
            throw new LockException("Document allready locked (id)! " + documentId);
         }

         documentDO.setLOCKEDBY(userName);
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document is Locked (id):"
                                  + documentId + " by user(id):" + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#update(java.lang.String, java.lang.String,
    *      java.lang.String, java.lang.String)
    */
   public void update(String documentId, String path, String userName, String foDocumentId)
      throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);

         String lockUserName = documentDO.getLOCKEDBY();
         if (lockUserName == null) {
            throw new LockException("Document isn't locked (id)! " + documentId);
         }

         if (!lockUserName.equals(userName)) {
            throw new LockException("User can't update document (id)! " + documentId);
         }

         File file = new File(path);
         if (!file.exists()) {
            throw new DocumentStoreException("File not found on Document Store path for document (id): "
                                             + documentId);
         }

         if (documentDO.getAUTOVERSIONABLE()) {
            DocumentVersionDO dvDO = createDocumentVersion(documentDO, userName);
            if (null == dvDO) {
               throw new DatabaseException("Document Version creation error!");
            }
            dvDO.save();
         }

         FoDocumentDO foDocumentRefDO = null;
         if (foDocumentId != null) {
            foDocumentRefDO = getFoDocumentRefDO(foDocumentId);
         }

         documentDO.setFILEPATH(path);
         documentDO.setLASTMODIFIEDDATE(new Timestamp(System.currentTimeMillis()));
         documentDO.setSIZE(new BigDecimal(file.length()));
         documentDO.setLASTMODIFIEDBY(userName);
         documentDO.setLOCKEDBY(null);
         documentDO.setFODOCUMENTREF(foDocumentRefDO);
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document is Updated (id):"
                                  + documentId + " by user(id):" + userName);
         }
      } catch (NumberFormatException e) {
         throw new DatabaseException(e);
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#getDocument(java.lang.String)
    */
   public Document getDocument(String documentId) throws BaseException {
      DocumentDO documentDO = getDocumentDOByNumber(documentId);
      if (null != logger) {
         logger.log(Log.DEBUG, "Returning document (id):" + documentId);
      }
      return new DocumentImpl(documentDO);
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#getDocumentVersion(java.lang.String)
    */
   public DocumentVersion getDocumentVersion(String documentVersionId)
      throws BaseException {
      DocumentVersionDO documentVersionDO = getDocumentVersionDOByNumber(documentVersionId);
      if (null != logger) {
         logger.log(Log.DEBUG, "Returning document version (id):" + documentVersionId);
      }
      return new DocumentVersionImpl(documentVersionDO);
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#delete(java.lang.String, java.lang.String)
    */
   public void delete(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);
         if (documentDO.getLOCKEDBY() != null || documentDO.getCHECKEDOUTBY() != null) {
            throw new LockException("Document is locked (id)! " + documentId);
         }
         if (documentDO.getDELETEDBY() != null) {
            throw new ActionNotAllowedException("Document allready deleted (id)! "
                                                + documentId);
         }

         documentDO.setDELETEDBY(userName);
         documentDO.setDELETEDDATE(new Timestamp(System.currentTimeMillis()));
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document is deleted (id):"
                                  + documentId + " by user " + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#create(java.lang.String, java.lang.String,
    *      java.lang.String, java.lang.String, java.lang.String, boolean,
    *      java.lang.String)
    */
   public String create(String documentName,
                        String documentPath,
                        String mimeType,
                        String userName,
                        String foDocumentId,
                        boolean isTemplate,
                        String templateId) throws BaseException {

      FoDocumentDO foDocumentRefDO = null;
      if (foDocumentId != null) {
         foDocumentRefDO = getFoDocumentRefDO(foDocumentId);
      }

      DocumentDO templateRefDO = null;
      if (templateId != null) {
         templateRefDO = getDocumentDOByNumber(templateId);
      }

      Document document = new DocumentImpl();
      document.setCurrentVersionNumber("1");
      document.setLastModifiedDate(System.currentTimeMillis());
      document.setCreatedDate(System.currentTimeMillis());
      document.setMimeType(mimeType);
      document.setName(documentName);
      if (null == documentPath) {
         throw new DocumentStoreException("Error while saving document content!");
      }
      File documentFile = new File(documentPath);
      if (documentFile == null || !documentFile.exists()) {
         throw new DocumentStoreException("Error while saving document content!");
      }

      document.setFilepath(documentPath);
      document.setSize(documentFile.length());
      document.setAutoversionable(true);
      document.setTemplate(isTemplate);
      try {
         DocumentDO documentDO = document.createDO();
         documentDO.setCREATEDBY(userName);
         documentDO.setLASTMODIFIEDBY(userName);
         documentDO.setFODOCUMENTREF(foDocumentRefDO);
         documentDO.setTEMPLATEREF(templateRefDO);
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document :"
                                  + documentName + "is created  by " + userName);
         }

         return documentDO.get_Handle();
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#getDocumentVersions(java.lang.String)
    */
   public DocumentVersion[] getDocumentVersions(String id) throws BaseException {
      DocumentVersionImpl[] documentVersArray = null;
      try {

         DocumentDO documentDO = getDocumentDOByNumber(id);
         if (documentDO == null) {
            throw new DatabaseException("Document not found (id) - " + id);
         }

         DocumentVersionQuery query = new DocumentVersionQuery();
         query.setQueryDOCUMENTOID(documentDO);

         DocumentVersionDO[] documentVersDOArray = query.getDOArray();
         if (null != documentVersDOArray) {
            documentVersArray = new DocumentVersionImpl[documentVersDOArray.length];
            for (int i = 0; i < documentVersDOArray.length; i++) {
               documentVersArray[i] = new DocumentVersionImpl(documentVersDOArray[i]);
            }
         } else {
            throw new DatabaseException("Error while loading document versions array for document (id) - "
                                        + id);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      }
      if (null != logger) {
         logger.log(Log.DEBUG, "Getting Document versions for Document (id):" + id);
      }
      return documentVersArray;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#getDocuments(boolean)
    */
   public Document[] getDocuments(boolean isTemplate) throws BaseException {
      DocumentImpl[] documentArray = null;
      try {
         DocumentQuery query = new DocumentQuery();
         query.setQueryDELETEDBY(null);
         query.setQueryISTEMPLATE(isTemplate);
         DocumentDO[] documentDOArray = query.getDOArray();
         if (null != documentDOArray) {
            documentArray = new DocumentImpl[documentDOArray.length];
            for (int i = 0; i < documentDOArray.length; i++) {
               documentArray[i] = new DocumentImpl(documentDOArray[i]);
            }
         } else {
            throw new DatabaseException("Error while loading documents!");
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      }
      if (null != logger) {
         logger.log(Log.DEBUG, "Getting Documents!");
      }
      return documentArray;
   }

   /**
    * Method that creates document version and increment version number
    * 
    * @param documentDO
    * @param userName
    * @return
    * @throws BaseException
    */
   private DocumentVersionDO createDocumentVersion(DocumentDO documentDO, String userName)
      throws BaseException {
      try {
         Document doc = new DocumentImpl(documentDO);
         DocumentVersion dv = (DocumentVersionImpl) doc.createVersion();
         DocumentVersionDO dvDO = dv.createVersionDO();
         dvDO.setDOCUMENTOID(documentDO);
         dvDO.setLASTMODIFIEDBY(userName);
         BigDecimal bd = documentDO.getCURRENTVERSIONNUMBER();
         int currentVersion = bd.intValue();
         currentVersion++;
         documentDO.setCURRENTVERSIONNUMBER(new BigDecimal(currentVersion));
         if (null != logger) {
            logger.log(Log.DEBUG, "Document version Created for document "
                                  + documentDO.get_OId() + " by user " + userName);
         }
         return dvDO;
      } catch (Exception e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#restoreDocumentVersion(java.lang.String,
    *      java.lang.String)
    */
   public void restoreDocumentVersion(String versionId, String userName)
      throws BaseException {
      DocumentVersion documentVersion = getDocumentVersion(versionId);
      try {
         String id = documentVersion.getDocument().getId();
         DocumentDO documentDO = getDocumentDO(id);
         if (null == documentDO) {
            throw new DatabaseException("Document not found (ID) " + id);
         }
         if (documentDO.getLOCKEDBY() != null || documentDO.getCHECKEDOUTBY() != null) {
            throw new LockException("Document is locked (ID)! " + id);
         }

         DocumentVersionDO dvDO = createDocumentVersion(documentDO, userName);
         documentDO.setFILEPATH(documentVersion.getFilepath());
         documentDO.setLASTMODIFIEDDATE(new Timestamp(documentVersion.getLastModifiedDate()));
         documentDO.setSIZE(new BigDecimal(documentVersion.getSize()));
         if (documentVersion.getFoDocumentRef()!=null)
            documentDO.oid_setFODOCUMENTREF(documentVersion.getFoDocumentRef().getId());
         documentDO.setLASTMODIFIEDBY(userName);
         documentDO.save();
         dvDO.save();

         if (null != logger) {
            logger.log(Log.DEBUG,
                       "Restoring Document Version (DOC_VERS_ID)",
                       documentVersion.getNumber());
         }
      } catch (Exception e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#getLogger()
    */
   public Log getLogger() {
      return logger;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#setLogger(org.enhydra.dm.api.loggers.Log)
    */
   public void setLogger(Log logger) {
      this.logger = logger;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#configure(java.util.Properties)
    */
   public void configure(Properties properties) throws BaseException {
      // do some property initialization
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#restore(java.lang.String, java.lang.String)
    */
   public void restore(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);
         if (documentDO.getDELETEDBY() == null) {
            throw new ActionNotAllowedException("Document Allready Restored (ID)! "
                                                + documentId);
         }

         documentDO.setDELETEDBY(null);
         documentDO.setDELETEDDATE(null);
         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Document is successfully restored(ID):"
                                  + documentId + " by user " + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#getDeleted(boolean)
    */
   public Document[] getDeleted(boolean isTemplate) throws BaseException {
      DocumentImpl[] documentArray = null;
      try {
         DocumentQuery query = new DocumentQuery();
         query.setQueryDELETEDBY(null, QueryBuilder.NOT_EQUAL);
         query.setQueryISTEMPLATE(isTemplate);
         DocumentDO[] documentDOArray = query.getDOArray();
         if (null != documentDOArray) {
            documentArray = new DocumentImpl[documentDOArray.length];
            for (int i = 0; i < documentDOArray.length; i++) {
               documentArray[i] = new DocumentImpl(documentDOArray[i]);
            }
         } else {
            throw new DatabaseException("Error while loading archive documents!");
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      }
      if (null != logger) {
         logger.log(Log.DEBUG, "Getting deleted documents from database!");
      }
      return documentArray;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#switchAutoVersionable(java.lang.String,
    *      java.lang.String)
    */
   public void switchAutoVersionable(String documentId, String userName)
      throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);
         if (documentDO.getLOCKEDBY() != null || documentDO.getCHECKEDOUTBY() != null) {
            throw new LockException("Document is locked (id)! " + documentId);
         }

         if (documentDO.getAUTOVERSIONABLE()) {
            documentDO.setAUTOVERSIONABLE(false);
         } else {
            documentDO.setAUTOVERSIONABLE(true);
         }

         documentDO.save();
         if (null != logger) {
            logger.log(Log.DEBUG, "Versionable switched for Document (id):"
                                  + documentId + " by user " + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }

   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#switchArchived(java.lang.String,
    *      java.lang.String)
    */
   public void switchArchived(String documentId, String userName) throws BaseException {
      try {
         DocumentDO documentDO = getDocumentDOByNumber(documentId);
         if (documentDO.getLOCKEDBY() != null || documentDO.getCHECKEDOUTBY() != null) {
            throw new LockException("Document is locked (id)! " + documentId);
         }

         if (documentDO.getARCHIVED()) {
            documentDO.setARCHIVED(false);
         } else {
            documentDO.setARCHIVED(true);
         }

         documentDO.save();

         if (null != logger) {
            logger.log(Log.DEBUG, "Archived switched for Document (id):"
                                  + documentId + " by user " + userName);
         }
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }

   }

   /**
    * Return DocumentDO object by document id
    * 
    * @param documentId
    * @return
    * @throws BaseException
    */
   private DocumentDO getDocumentDO(String documentId) throws BaseException {
      DocumentDO documentDO = DocumentDO.createExisting(documentId);
      if (documentDO == null) {
         throw new DatabaseException("Document not found (ID) - " + documentId);
      }
      return documentDO;
   }

   /**
    * Return DocumentDO object by document number
    * 
    * @param documentNumber
    * @return
    * @throws BaseException
    */
   private DocumentDO getDocumentDOByNumber(String documentNumber) throws BaseException {

      DocumentDO documentDO = null;
      DocumentQuery dq = new DocumentQuery();
      try {
         dq.requireUniqueInstance();
         dq.setQueryLOGNUM(Integer.parseInt(documentNumber));
         documentDO = dq.getNextDO();

      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      }
      if (documentDO == null) {
         throw new DatabaseException("Document not found (NUMBER) - " + documentNumber);
      }

      return documentDO;
   }

   /**
    * Search for DocumentVersionDO by documentVersionNumber
    * 
    * @param documentVersionNumber
    * @return
    * @throws BaseException
    */
   private DocumentVersionDO getDocumentVersionDOByNumber(String documentVersionNumber)
      throws BaseException {

      DocumentVersionDO documentVersionDO = null;
      DocumentVersionQuery dvq = new DocumentVersionQuery();
      try {
         dvq.requireUniqueInstance();
         dvq.setQueryLOGNUM(Integer.parseInt(documentVersionNumber));
         documentVersionDO = dvq.getNextDO();

      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      }
      if (documentVersionDO == null) {
         throw new DatabaseException("DocumentVersion not found (NUMBER) - "
                                     + documentVersionNumber);
      }

      return documentVersionDO;
   }

   /**
    * Search for FoDocumentDO by foDocumentId
    * 
    * @param foDocumentId
    * @return
    * @throws BaseException
    */
   private FoDocumentDO getFoDocumentRefDO(String foDocumentId) throws BaseException {
      FoDocumentDO foDocumentDO = FoDocumentDO.createExisting(foDocumentId);
      if (foDocumentDO == null) {
         throw new DatabaseException("FoDocument not found (id) - " + foDocumentId);
      }
      return foDocumentDO;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentManager#createVersion(java.lang.String,
    *      java.lang.String)
    */
   public void createVersion(String documentId, String userName) throws BaseException {
      DocumentDO documentDO = getDocumentDOByNumber(documentId);
      if (documentDO == null) {
         throw new DatabaseException("Document not found (id) - " + documentId);
      }
      DocumentVersionDO dvDO = createDocumentVersion(documentDO, userName);
      if (dvDO == null) {
         throw new DatabaseException("Can't create document version for document (id) - "
                                     + documentId);
      }
      try {
         dvDO.save();
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }
}
