/**
 * 
 */
package org.enhydra.dm.business;

import java.math.BigDecimal;
import java.sql.Timestamp;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.DocumentVersion;
import org.enhydra.dm.api.FoDocument;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.DatabaseException;
import org.enhydra.dm.data.DocumentVersionDO;
import org.enhydra.dm.data.FoDocumentDO;
import org.enhydra.dm.util.Numerator;

/**
 * Implementation of DocumentVersion interface.
 * 
 * @author Slobodan Vujasinovic
 */
public class DocumentVersionImpl implements DocumentVersion {

   private static final String DOCUMENTVERSION_NUMBER_COLUMN_NAME = "DOCUMENTVERSION";

   String id;

   String number;

   Document document = null;

   String lastModifiedBy = null;

   String filepath;

   long lastModifiedDate;

   String versionNumber;

   long versionDate;

   long size;

   FoDocument foDocumentRef = null;

   DocumentVersionImpl() {

   }

   public DocumentVersionImpl(DocumentVersionDO documentVersionDO) throws BaseException {
      try {
         setId(documentVersionDO.get_Handle());
         setFilepath(documentVersionDO.getFILEPATH());
         setLastModifiedDate(documentVersionDO.getLASTMODIFIEDDATE().getTime());
         setVersionNumber(documentVersionDO.getVERSIONNUMBER().toString());
         setVersionDate(documentVersionDO.getVERSIONDATE().getTime());
         setDocument(new DocumentImpl(documentVersionDO.getDOCUMENTOID()));
         setLastModifiedBy(documentVersionDO.getLASTMODIFIEDBY());

         setNumber(String.valueOf(documentVersionDO.getLOGNUM()));
         FoDocumentDO foDocDO = documentVersionDO.getFODOCUMENTREF();
         if (foDocDO != null) {
            setFoDocumentRef(new FoDocumentImpl(foDocDO));
         }
         setSize(documentVersionDO.getSIZE().intValue());
      } catch (Exception e) {
         throw new DatabaseException(e);
      }

   }

   public DocumentVersionDO createVersionDO() throws BaseException {
      try {
         DocumentVersionDO documentVersionDO = DocumentVersionDO.createVirgin();
         documentVersionDO.setVERSIONNUMBER(new BigDecimal(this.getVersionNumber()));
         documentVersionDO.setVERSIONDATE(new Timestamp(this.getVersionDate()));
         documentVersionDO.setFILEPATH(this.getFilepath());
         documentVersionDO.setLASTMODIFIEDDATE(new Timestamp(this.getLastModifiedDate()));
         documentVersionDO.setSIZE(new BigDecimal(getSize()));
         if (getFoDocumentRef()!=null)
            documentVersionDO.oid_setFODOCUMENTREF(getFoDocumentRef().getId());
         documentVersionDO.setLOGNUM(Numerator.getNext(DOCUMENTVERSION_NUMBER_COLUMN_NAME)
            .intValue());

         return documentVersionDO;
      } catch (Exception e) {
         throw new DatabaseException(e);
      }
   }

   public Document getDocument() {
      return document;
   }

   public void setDocument(Document document) {
      this.document = document;
   }

   public String getFilepath() {
      return filepath;
   }

   public void setFilepath(String filepath) {
      this.filepath = filepath;
   }

   public String getLastModifiedBy() {
      return lastModifiedBy;
   }

   public void setLastModifiedBy(String lastModifiedBy) {
      this.lastModifiedBy = lastModifiedBy;
   }

   public long getLastModifiedDate() {
      return lastModifiedDate;
   }

   public void setLastModifiedDate(long lastModifiedDate) {
      this.lastModifiedDate = lastModifiedDate;
   }

   public String getNumber() {
      return number;
   }

   public void setNumber(String number) {
      this.number = number;
   }

   public String getId() {
      return id;
   }

   public void setId(String id) {
      this.id = id;
   }

   public long getSize() {
      return size;
   }

   public void setSize(long size) {
      this.size = size;
   }

   public long getVersionDate() {
      return versionDate;
   }

   public void setVersionDate(long versionDate) {
      this.versionDate = versionDate;
   }

   public String getVersionNumber() {
      return versionNumber;
   }

   public void setVersionNumber(String versionNumber) {
      this.versionNumber = versionNumber;
   }

   public FoDocument getFoDocumentRef() {
      return this.foDocumentRef;
   }

   public void setFoDocumentRef(FoDocument foDocumentRef) {
      this.foDocumentRef = foDocumentRef;
   }

}
