package org.enhydra.dm.business;

import org.enhydra.dm.api.FoDocument;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.data.FoDocumentDO;

import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;

/**
 * Implementation of FoDocument interface.
 * 
 * @author Zorica Dudarin
 */

public class FoDocumentImpl implements FoDocument {

   String id;

   String userPassword;

   String ownerPassword;

   boolean allowPrint;

   boolean allowCopyContent;

   boolean allowEditContent;

   boolean allowEditAnnotation;

   public FoDocumentImpl() {

   }

   public FoDocumentImpl(FoDocumentDO foDocumentDO) throws BaseException {
      try {
         setId(foDocumentDO.get_OId().toString());
         setUserPassword(foDocumentDO.getUSERPASSWORD());
         setOwnerPassword(foDocumentDO.getOWNERPASSWORD());
         setAllowPrint(foDocumentDO.getALLOWPRINT());
         setAllowCopyContent(foDocumentDO.getALLOWCOPYCONTENT());
         setAllowEditContent(foDocumentDO.getALLOWEDITCONTENT());
         setAllowEditAnnotation(foDocumentDO.getALLOWEDITANNOTATIONS());
      } catch (Exception e) {
         throw new BaseException(e);
      }
   }

   public FoDocumentImpl(String id) throws BaseException {
      try {
         setId(id);
      } catch (Exception e) {
         throw new BaseException(e);
      }
   }

   public String getId() {
      return id;
   }

   public void setId(String id) {
      this.id = id;
   }

   public String getUserPassword() {
      return userPassword;
   }

   public void setUserPassword(String userPassword) {
      this.userPassword = userPassword;
   }

   public String getOwnerPassword() {
      return ownerPassword;
   }

   public void setOwnerPassword(String ownerPassword) {
      this.ownerPassword = ownerPassword;
   }

   public boolean isAllowPrint() {
      return allowPrint;
   }

   public void setAllowPrint(boolean allowPrint) {
      this.allowPrint = allowPrint;
   }

   public boolean isAllowCopyContent() {
      return allowCopyContent;
   }

   public void setAllowCopyContent(boolean allowCopyContent) {
      this.allowCopyContent = allowCopyContent;
   }

   public boolean isAllowEditContent() {
      return allowEditContent;
   }

   public void setAllowEditContent(boolean allowEditContent) {
      this.allowEditContent = allowEditContent;
   }

   public boolean isAllowEditAnnotation() {
      return allowEditAnnotation;
   }

   public void setAllowEditAnnotation(boolean allowEditAnnotation) {
      this.allowEditAnnotation = allowEditAnnotation;
   }

   public FoDocumentDO createDO() throws BaseException {
      try {
         FoDocumentDO foDocumentDo = FoDocumentDO.createVirgin();
         foDocumentDo.setUSERPASSWORD(this.userPassword);
         foDocumentDo.setOWNERPASSWORD(this.ownerPassword);
         foDocumentDo.setALLOWPRINT(this.allowPrint);
         foDocumentDo.setALLOWCOPYCONTENT(this.allowCopyContent);
         foDocumentDo.setALLOWEDITCONTENT(this.allowEditContent);
         foDocumentDo.setALLOWEDITANNOTATIONS(this.allowEditAnnotation);

         return foDocumentDo;
      } catch (DatabaseManagerException e) {
         throw new BaseException(e);
      } catch (ObjectIdException e) {
         throw new BaseException(e);
      } catch (DataObjectException e) {
         throw new BaseException(e);
      }
   }
}
