package org.enhydra.dm.business;

import java.sql.SQLException;
import java.util.Iterator;
import java.util.Map;

import org.enhydra.dm.api.FoDocument;
import org.enhydra.dm.api.FoDocumentManager;
import org.enhydra.dm.api.FoDocumentParam;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.business.exceptions.DatabaseException;
import org.enhydra.dm.data.FoDocumentDO;
import org.enhydra.dm.data.FoDocumentParamDO;
import org.enhydra.dm.data.FoDocumentParamQuery;
import org.enhydra.dm.data.FoDocumentQuery;

import com.lutris.appserver.server.sql.DBRowUpdateException;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.dods.builder.generator.query.NonUniqueQueryException;
import com.lutris.dods.builder.generator.query.QueryException;
import com.lutris.dods.builder.generator.query.RefAssertionException;

public class FoDocumentManagerImpl implements FoDocumentManager {

   private Log logger = null;

   /**
    * Empty constructor
    */
   public FoDocumentManagerImpl() {

   }

   /**
    * @param parameters
    * @param userPwd
    * @param ownerPwd
    * @param allowPrint
    * @param allowCopyContent
    * @param allowEditContent
    * @param allowEditAnnotations
    * @return foDocumentId
    * @throws BaseException
    */
   public String create(Map parameters,
                        String userPwd,
                        String ownerPwd,
                        boolean allowPrint,
                        boolean allowCopyContent,
                        boolean allowEditContent,
                        boolean allowEditAnnotation) throws BaseException {
      if (null != getLogger()) {
         getLogger().log(Log.DEBUG, "Create action for foDocument.");
      }

      try {

         FoDocument foDocument = new FoDocumentImpl();
         foDocument.setUserPassword(userPwd);
         foDocument.setOwnerPassword(ownerPwd);
         foDocument.setAllowPrint(allowPrint);
         foDocument.setAllowCopyContent(allowCopyContent);
         foDocument.setAllowEditContent(allowEditContent);
         foDocument.setAllowEditAnnotation(allowEditAnnotation);

         FoDocumentDO foDocumentDO = null;

         foDocumentDO = foDocument.createDO();
         foDocumentDO.save();

         Iterator entries = parameters.entrySet().iterator();

         while (entries.hasNext()) {
            try {
               Map.Entry me = (Map.Entry) entries.next();
               String name = (String) me.getKey();
               String value = (String) me.getValue();
               if (value != null) {
                  FoDocumentParam foDocumentParam = new FoDocumentParamImpl();
                  foDocumentParam.setName(name);
                  foDocumentParam.setContent(value);
                  FoDocumentParamDO fdpDO =  foDocumentParam.createDO();
                  fdpDO.setFODOCUMENTREF(foDocumentDO);
                  fdpDO.save();
               }
            } catch (Exception ex) {
               throw new DatabaseException("Failed to create FoDocument Parameter entry!");
            }
         }

         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "foDocument is created.");
         }
         return foDocumentDO.get_OId().toString();

      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /**
    * Getter for logger
    * 
    * @return
    */
   public Log getLogger() {
      return logger;
   }

   /**
    * Setter for logger
    * 
    * @param logger
    */
   public void setLogger(Log logger) {
      this.logger = logger;
   }

   /**
    * @param id
    * @return
    * @throws BaseException
    */
   public FoDocument getFoDocumentById(String id) throws BaseException {
      FoDocument foDocument = null;
      foDocument = new FoDocumentImpl(getFoDocumentDO(id));

      return foDocument;
   }

   public FoDocumentParam[] getFoDocumentParametersById(String id) throws BaseException {
      FoDocumentParamDO [] foDocumentParamDOArray = null;
      FoDocumentParam [] foDocumentParamArray = null;
      FoDocumentParamQuery foDocumentParamQuery = new FoDocumentParamQuery();;
      
      try {
         foDocumentParamQuery.setQueryFODOCUMENTREF(getFoDocumentDO(id));
         foDocumentParamDOArray = foDocumentParamQuery.getDOArray();

      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      }
      if (foDocumentParamDOArray == null) {
         throw new DatabaseException("FoDocument parameters not found (ID) - " + id);
      }
      
      foDocumentParamArray = new FoDocumentParam[foDocumentParamDOArray.length];
      for (int i=0;i<foDocumentParamDOArray.length;i++){
         foDocumentParamArray[i]= new FoDocumentParamImpl(foDocumentParamDOArray[i]);
      }

      return foDocumentParamArray;
   }
   
   /**
    * @param id
    * @return
    * @throws BaseException
    */
   private FoDocumentDO getFoDocumentDO(String id) throws BaseException {
      if (null != getLogger()) {
         getLogger().log(Log.DEBUG, "Getting the foDocuments from database!");
      }

      FoDocumentDO foDocumentDO = FoDocumentDO.createExisting(id);

      if (foDocumentDO == null) {
         throw new DatabaseException("FoDocument not found (ID) - " + id);
      }

      return foDocumentDO;
   }
}
