package org.enhydra.dm.business;

import java.sql.SQLException;
import java.util.Properties;

import org.enhydra.dm.api.AppUser;
import org.enhydra.dm.api.UserManager;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.business.exceptions.DatabaseException;
import org.enhydra.dm.data.AppUserDO;
import org.enhydra.dm.data.AppUserQuery;

import com.lutris.appserver.server.sql.DBRowUpdateException;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.dods.builder.generator.query.NonUniqueQueryException;
import com.lutris.dods.builder.generator.query.QueryException;
import com.lutris.dods.builder.generator.query.RefAssertionException;

/**
 * Default implementation of UserManager interface. Used for FORM authentication.
 * 
 * @author Svjetlana Milidrag
 */

public class UserManagerImpl implements UserManager {

   private Log logger = null;

   private final String DEFAULT_ROLE = "admin";

   public UserManagerImpl() {

   }

   /**
    * Check if username already exists.
    * 
    * @param username
    * @return
    * @throws BaseException
    */
   private boolean usernameAlreadyExists(String username) throws BaseException {
      if (null != logger) {
         logger.log(Log.DEBUG, "Check for username " + username + " existing! ");
      }
      try {
         AppUserQuery query = new AppUserQuery();
         query.setQueryUSERNAME(username);

         AppUserDO userDO = query.getNextDO();
         if (null != userDO) {
            return true;
         }

      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      }
      return false;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.UserManager#getUserByUsername(java.lang.String)
    */
   public AppUser getUserByUsername(String username) throws BaseException {
      if (null != logger) {
         logger.log(Log.DEBUG, "Searching for user by username ! ", username);
      }
      try {
         AppUserQuery query = new AppUserQuery();
         query.setQueryUSERNAME(username);

         AppUserDO userDO = query.getNextDO();
         if (null != userDO) {

            return new AppUserImpl(userDO);
         } else {
            throw new DatabaseException("User "
                                        + username + " doesn't exist in database!");
         }

      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (NonUniqueQueryException e) {
         throw new DatabaseException(e);
      }

   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.UserManager#getUserById(java.lang.String)
    */
   public AppUser getUserById(String id) throws BaseException {
      if (null != logger) {
         logger.log(Log.DEBUG, "Searching for user by id ! ", id);
      }
      try {
         AppUserDO userDO = AppUserDO.createExisting(id);

         if (null != userDO) {
            AppUser user = new AppUserImpl(userDO);
            return user;
         } else {
            throw new DatabaseException("User doesn't exist for id: " + id + ";");
         }
      } catch (Exception ex) {
         throw new DatabaseException(ex.getMessage());
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.UserManager#createUser(java.lang.String, java.lang.String,
    *      java.lang.String, java.lang.String)
    */
   public boolean createUser(String firstname,
                             String lastname,
                             String username,
                             String password) throws BaseException {

      if (null != logger) {
         logger.log(Log.DEBUG, "User name: "
                               + firstname + " : " + lastname + " with username: "
                               + username + " creation ! ");
      }
      if (usernameAlreadyExists(username)) {
         return false;
      }
      AppUser user = new AppUserImpl();
      user.setFirstname(firstname);
      user.setLastname(lastname);
      user.setUsername(username);
      user.setPassword(password);
      user.setRole(DEFAULT_ROLE);

      try {
         user.createDO().save();
         return true;
      } catch (DatabaseManagerException e) {
         throw new DatabaseException(e);
      } catch (DataObjectException e) {
         throw new DatabaseException(e);
      } catch (RefAssertionException e) {
         throw new DatabaseException(e);
      } catch (DBRowUpdateException e) {
         throw new DatabaseException(e);
      } catch (QueryException e) {
         throw new DatabaseException(e);
      } catch (SQLException e) {
         throw new DatabaseException(e);
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.UserManager#configure(java.util.Properties)
    */
   public void configure(Properties properties) throws BaseException {

   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.UserManager#getLogger()
    */
   public Log getLogger() {
      return logger;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.UserManager#setLogger(org.enhydra.dm.api.loggers.Log)
    */
   public void setLogger(Log logger) {
      this.logger = logger;
   }

}
