/*
 * Enhydra super-servlet
 */

package org.enhydra.dm;

import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.enhydra.dm.api.DocumentManager;
import org.enhydra.dm.api.DocumentStore;
import org.enhydra.dm.api.FoDocumentManager;
import org.enhydra.dm.api.UserManager;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.api.util.DesEncription;
import org.enhydra.dm.business.DocumentManagerImpl;
import org.enhydra.dm.business.DocumentStoreImpl;
import org.enhydra.dm.business.FoDocumentManagerImpl;
import org.enhydra.dm.business.UserManagerImpl;
import org.enhydra.dm.util.Numerator;
import org.enhydra.dm.util.TransformUtility;
import org.enhydra.dm.util.filesys.PropertyChangeListenerImpl;

import com.lutris.appserver.server.Application;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.StandardApplication;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;

/**
 * The application object. Application-wide data would go here.
 */
public class EnhydraDM extends StandardApplication {

   public static String userTransactionJNDIPath;

   public static final String defaultUserTransactionJNDIPath = "java:comp/env/UserTransaction";

   public static final String defaultLogImpl = "org.enhydra.dm.api.loggers.DefaultLog";

   public static final String defaultDocumentManagerImpl = DocumentManagerImpl.class.getName();

   public static final String defaultDocumentStoreImpl = DocumentStoreImpl.class.getName();

   public static final String defaultUserManagerImpl = UserManagerImpl.class.getName();

   public static String allowedExtensionsForUpload;

   public static int allowedFileSizeForUpload;

   public static String previewerURL;

   public static String previewerXSL;

   public static final String defaultDesEncriptionImpl = null;

   public static Log logger;

   /**
    * Descriptor / Encriptor
    */
   public static DesEncription desenc = null;

   public static Application myself = null;

   /**
    * Document Manager
    */
   private DocumentManager documentManager = null;

   /**
    * Document Manager
    */
   private DocumentStore documentStore = null;

   /**
    * App Users Manager
    */
   private UserManager userManager = null;

   /**
    * Label properties files path
    */
   private String propFilesPath = null;

   /**
    * xsl and properties file refresh period set up in seconds.
    */
   private long fileRefreshPeriod = 0;

   /**
    * Key is properties file name, value is appropriate
    * <code>PropetiesChangeListener</code>
    */
   private Map propsMap = new HashMap();

   /**
    * FoDocument Manager
    */
   private FoDocumentManager foDocumentManager = null;

   /*
    * A few methods you might want to add to. See StandardApplication for more details.
    */
   public void startup(Config appConfig) throws ApplicationException {
      super.startup(appConfig);
      myself = this;
      // Calculating default document store path
      Properties props = null;

      try {
         props = readConfig(config);
         initLogger(props);
         userTransactionJNDIPath = appConfig.getString("DatabaseManager.defaults.XATransactionManagerLookupName",
                                                       defaultUserTransactionJNDIPath);

      } catch (ConfigException e) {
         userTransactionJNDIPath = defaultUserTransactionJNDIPath;
         logger = Log.getInstance();

      }
      if (null != logger) {
         logger.log(Log.INFORMATION, "Logger for EnhydraDM is successfully initialized!");
      }

      initDesEncriptor(props);
      initUserManager(props);
      initDocumentManager(props);
      initDocumentStore(props);
      setFileRefreshPeriod(appConfig);
      initTransformUtility(props);
      initPropFiles(appConfig);
      initFoDocumentManager(appConfig);
      setAllowedExtensionsForUpload(appConfig);
      setAllowedFileSizeForUpload(appConfig);
      setPreviewerURL(appConfig);
      setPreviewerXSL(appConfig);
      initNumerator(props);

   }

   private void setAllowedExtensionsForUpload(Config appConfig) {
      try {
         String temp = appConfig.getString("Upload.Extensions");
         if (temp != null) {
            allowedExtensionsForUpload = temp;
         }
      } catch (Exception ex) {
      }
      if (allowedExtensionsForUpload == null) {
         if (null != logger) {
            logger.log(Log.WARNING,
                       "Allowed extensions for upload setup failed. Default value is set.");
         }
         allowedExtensionsForUpload = "doc, docx, xls, xslx, ppt, pptx, pps, ppsx, jpeg, msg, pdf, zip, gif, bmp, txt, tif, png";
      }
   }

   private void setAllowedFileSizeForUpload(Config appConfig) {
      try {
         int temp = appConfig.getInt("Upload.FileSize");
         if (temp > 0) {
            allowedFileSizeForUpload = temp;
         }
      } catch (Exception ex) {
      }
      if (allowedFileSizeForUpload <= 0) {
         if (null != logger) {
            logger.log(Log.WARNING,
                       "Allowed file size for upload setup failed. Default value is set.");
         }
         allowedFileSizeForUpload = 2000000;
      }

   }

   private void setPreviewerURL(Config appConfig) {
      try {
         String temp = appConfig.getString("Previewer.URL");
         if (temp != null && !temp.equals("")) {
            previewerURL = temp;
         }
      } catch (Exception ex) {
      }
      if (previewerURL == null) {
         if (null != logger) {
            logger.log(Log.INFORMATION, "No previewer URL is set. Using default value.");
         }
         previewerURL = "";
      }
   }

   private void setPreviewerXSL(Config appConfig) {
      try {
         String temp = appConfig.getString("Previewer.XSL");
         if (temp != null && !temp.equals("")) {
            previewerXSL = temp;
         }
      } catch (Exception ex) {
      }
      if (previewerXSL == null) {
         if (null != logger) {
            logger.log(Log.INFORMATION, "No previewer XSL is set. Using default value.");
         }
         previewerXSL = "preview";
      }
   }

   private void initLogger(Properties props) throws ApplicationException {
      String logProviderName = Log.class.getName();
      try {
         String logProvider = props.getProperty(logProviderName, defaultLogImpl);
         try {
            logger = (Log) Class.forName(logProvider).newInstance();
            Log.setLogger(logger);
         } catch (Exception ex) {
            logger = Log.getInstance();
         }
      } catch (Exception e) {
         throw new ApplicationException("Init DesEncriptor exception!", e);
      }
   }

   private void initUserManager(Properties props) throws ApplicationException {
      try {
         String userManagerImplClassName = props.getProperty(UserManager.class.getName(),
                                                             defaultUserManagerImpl);
         userManager = (UserManager) Class.forName(userManagerImplClassName)
            .newInstance();
         if (null != logger) {
            userManager.setLogger(logger);
         }
      } catch (Exception e) {
         throw new ApplicationException("Init UserManager exception", e);
      }
   }

   private void initDocumentManager(Properties props) throws ApplicationException {

      try {

         String documentManagerImplClassName = props.getProperty(DocumentManager.class.getName(),
                                                                 defaultDocumentManagerImpl);
         documentManager = (DocumentManager) Class.forName(documentManagerImplClassName)
            .newInstance();
         if (null != logger) {
            documentManager.setLogger(logger);
         }
         documentManager.configure(props);

      } catch (Exception e) {
         throw new ApplicationException("Init DocumentaManager exception", e);
      }
   }

   private void initDocumentStore(Properties props) throws ApplicationException {

      try {
         String documentStoreImplClassName = props.getProperty(DocumentStore.class.getName(),
                                                               defaultDocumentStoreImpl);
         documentStore = (DocumentStore) Class.forName(documentStoreImplClassName)
            .newInstance();
         if (null != logger) {
            documentStore.setLogger(logger);
         }
         documentStore.configure(props);
      } catch (Exception e) {
         throw new ApplicationException("Init DocumentaManager exception", e);
      }
   }

   private void initDesEncriptor(Properties props) throws ApplicationException {
      try {

         String desEncriptionImplClass = props.getProperty(DesEncription.class.getName(),
                                                           defaultDesEncriptionImpl);
         if (null != desEncriptionImplClass) {
            desenc = (DesEncription) Class.forName(desEncriptionImplClass).newInstance();
            if (null != desenc) {
               desenc.setLogger(logger);
               desenc.configure(props);
            } else {
               if (null != logger) {
                  logger.log(Log.WARNING, "DesEncription implementation can't be found!");
               }
            }
         }

      } catch (Exception ex) {
         throw new ApplicationException("Init DesEncriptor exception!", ex);
      }
   }

   private void initNumerator(Properties props) throws ApplicationException {
      try {
         Numerator.setUserTransactionJNDIPath(userTransactionJNDIPath);
         Numerator.configure(props);
      } catch (Exception ex) {
         throw new ApplicationException("Init Numerator exception!", ex);
      }

   }

   public boolean requestPreprocessor(HttpPresentationComms comms) throws Exception {
      return super.requestPreprocessor(comms);
   }

   /**
    * This is an optional function, used only by the Multiserver's graphical
    * administration. This bit of HTML appears in the status page for this application.
    * You could add extra status info, for example a list of currently logged in users.
    * 
    * @return HTML that is displayed in the status page of the Multiserver.
    */
   public String toHtml() {
      return "This is <I>EnhydraDM</I> sample application!";
   }

   public DocumentManager getDocumentManager() {
      return documentManager;
   }

   public void setDocumentManager(DocumentManager documentManager) {
      this.documentManager = documentManager;
   }

   public DocumentStore getDocumentStore() {
      return documentStore;
   }

   public void setDocumentStore(DocumentStore documentStore) {
      this.documentStore = documentStore;
   }

   private Properties readConfig(Config config) throws ConfigException {
      Properties props = new Properties();
      try {
         String[] leafKeys = config.leafKeys();
         int size = leafKeys.length;

         for (int i = 0; i < size; i++) {
            String leafKey = leafKeys[i];
            Object value = config.get(leafKeys[i]);
            if (value.getClass().isArray()) {
               int len = Array.getLength(value);
               String valueArray = Array.get(value, 0).toString();
               for (int j = 1; j < len; j++) {
                  valueArray = valueArray + "," + Array.get(value, j).toString();
               }
               props.put(leafKey + "[]", valueArray);
            } else {
               if (value instanceof javax.sql.DataSource) {
                  props.put(leafKey, value.toString());
               } else {
                  props.put(leafKey, value);
               }
            }

         }

      } catch (Exception ex) {
         throw new ConfigException(ex);
      }
      return props;
   }

   /**
    * Method load fileRefreshPeriod from <code>web.xml</code> file. If
    * <code>fileRefreshPeriod</code> is 0 or negative number or it isn't set up
    * application won't reload *.properties and *.xsl files at runtime.
    * 
    * @param appConfig
    */
   private void setFileRefreshPeriod(Config appConfig) {
      try {
         int temp = appConfig.getInt("Application.FileRefreshPeriod");
         if (temp < 0) {
            if (null != logger) {
               logger.log(Log.INFORMATION,
                          "File refresh period setup failed. Negative period value:"
                                + temp);
            }
            fileRefreshPeriod = 0;
         } else {
            fileRefreshPeriod = temp * 1000;
         }
      } catch (Exception ex) {
         if (null != logger) {
            logger.log(Log.INFORMATION, "File refresh period setup failed.", ex);
         }
      }
   }

   /**
    * Method finds fiels and folders on the file sistem and make transformers, necessary
    * for making pdf file from editor content.
    * 
    * @param properties
    * @throws ApplicationException
    */
   private void initTransformUtility(Properties properties) throws ApplicationException {
      try {
         TransformUtility.configure(properties);
      } catch (Exception ex) {
         throw new ApplicationException("Init TransformUtility exception!", ex);
      }
   }

   /**
    * Method finds fodler with proerties files on the file sistem.
    * 
    * @param appConfig
    * @throws ApplicationException
    */
   private void initPropFiles(Config appConfig) throws ApplicationException {
      try {
         propFilesPath = appConfig.getString("Application.PropPath");
      } catch (ConfigException e) {
         throw new ApplicationException(e);
      }

   }

   /**
    * In <code>propsMap</code> map, method try to find appropriate
    * <code>propertyChangeListener</code> for proceeded file name. If it doesn't exist,
    * method try to make new <code>propertyChangeListener</code>. If properties file
    * for proceeded file name doesn't exist, it repeat this procedure for proceeded
    * default properties file name. Using <code>propertyChangeListener</code>, metod
    * get appropriate properties.
    * 
    * @param fileName - proerties file name
    * @param defaultFile - default proerties file name
    * @return <code>Properties</code> object
    */
   public Properties getProperties(String fileName, String defaultFile) {
      PropertyChangeListenerImpl propertyChangeListener = null;
      propertyChangeListener = (PropertyChangeListenerImpl) propsMap.get(fileName);

      String propFilePath = null;
      if (propertyChangeListener == null) {
         propFilePath = propFilesPath + fileName + ".properties";
         try {
            propertyChangeListener = new PropertyChangeListenerImpl(propFilePath,
                                                                    fileRefreshPeriod,
                                                                    logger);
            propsMap.put(fileName, propertyChangeListener);
         } catch (Exception e) {
            propertyChangeListener = (PropertyChangeListenerImpl) propsMap.get(defaultFile);

            if (propertyChangeListener != null) {
               return propertyChangeListener.getProperties();
            }

            propFilePath = propFilesPath + defaultFile + ".properties";
            try {
               propertyChangeListener = new PropertyChangeListenerImpl(propFilePath,
                                                                       fileRefreshPeriod,
                                                                       logger);
               propsMap.put(defaultFile, propertyChangeListener);
            } catch (Exception ex) {
               propertyChangeListener = null;
            }
         }
      }

      if (propertyChangeListener == null)
         return null;
      return propertyChangeListener.getProperties();
   }

   private void initFoDocumentManager(Config appConfig) throws ApplicationException {

      try {
         String foDocumentManagerImplClassName = config.getString(FoDocumentManager.class.getName(),
                                                                  FoDocumentManagerImpl.class.getName());
         foDocumentManager = (FoDocumentManager) Class.forName(foDocumentManagerImplClassName)
            .newInstance();
         if (null != logger) {
            foDocumentManager.setLogger(logger);
         }
      } catch (Exception e) {
         throw new ApplicationException("Init FoDocumentaManager exception", e);
      }
   }

   public FoDocumentManager getFoDocumentManager() {
      return foDocumentManager;
   }

   public void setFoDocumentManager(FoDocumentManager foDocumentManager) {
      this.foDocumentManager = foDocumentManager;
   }

   public UserManager getUserManager() {
      return userManager;
   }

   public void setUserManager(UserManager userManager) {
      this.userManager = userManager;
   }
}
