package org.enhydra.dm.presentation;

import java.util.Enumeration;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.DocumentManager;
import org.enhydra.dm.api.DocumentStore;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.xml.xmlc.html.HTMLObjectImpl;
import org.w3c.dom.Element;
import org.w3c.dom.html.HTMLInputElement;

import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;

public abstract class Base implements HttpPresentation {

   /**
    * User transaction
    */
   protected UserTransaction transaction = null;

   /**
    * Document Manager
    */
   protected DocumentManager documentManager = null;

   /**
    * Document Store
    */
   protected DocumentStore documentStore = null;

   public abstract void run(HttpPresentationComms comms) throws Exception;

   /**
    * Method initializes DB transation instance
    */
   protected void initTransaction() throws Exception {
      Context ctx = new InitialContext();
      transaction = (UserTransaction) ctx.lookup(EnhydraDM.userTransactionJNDIPath);
      if (transaction != null) {
         transaction.begin();
      }
   }

   /**
    * Method insures User presence
    */
   protected String getUser(HttpPresentationComms comms) throws Exception {
      String user = comms.request.getRemoteUser();
      return user;
   }

   /**
    * Method commits DB transation instance
    */
   protected void commitTransaction() throws Exception {
      if (transaction != null) {
         transaction.commit();
      }

   }

   /**
    * Method rollback's DB transation instance
    */
   protected void rollbackTransaction() throws Exception {
      if (transaction != null) {
         transaction.rollback();
      }
   }

   /**
    * Method sets lasbels names and button captions on proceeded page. 
    * It read label names fron proceeded properties object. 
    * 
    * @param prop - properties object with label names and button captions
    * @param objectHTML - HTML page
    * @throws Exception
    */
   protected void setI18n(Properties prop, HTMLObjectImpl objectHTML) throws Exception {
      String key = null;
      String val = null;
      Element element = null;

      Enumeration keys = prop.keys();

      if (prop != null) {
         while (keys.hasMoreElements()) {
            key = (String) keys.nextElement();
            val = prop.getProperty(key);
            element = objectHTML.getElementById(key);
            if (val != null && element != null) {
               if (element instanceof HTMLInputElement) {
                  element.setAttribute("value", val);
               } else {
                  element.setTextContent(val);
               }
            }
         }
      } else {
         throw new BaseException("Properties object is empty!");
      }
   }
}
