package org.enhydra.dm.presentation;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.Document;
import org.enhydra.dm.util.EnhydraDMConstants;

import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.logging.LogChannel;
import com.lutris.logging.Logger;

public class Details extends Base {

   /**
    * Corresponding XMLC object
    */
   private DetailsHTML page;

   /*
    * Function needed in order to be a presentation object.
    */
   public void run(HttpPresentationComms comms) throws Exception {
      page = (DetailsHTML) comms.xmlcFactory.create(DetailsHTML.class);

      comms.request.getHttpServletRequest().setCharacterEncoding(EnhydraDMConstants.ENCODING);

      String documentId = comms.request.getParameter(EnhydraDMConstants.DOCUMENT_ID);

      documentManager = ((EnhydraDM) comms.application).getDocumentManager();

      SimpleDateFormat sdf = new SimpleDateFormat(EnhydraDMConstants.DATE_FORMAT);

      boolean doCommit = true;
      try {
         initTransaction();
         String user = getUser(comms);

         page.setTextUser(user);

         page.getElementLogoutLink().setHref(EnhydraDMConstants.NAVIG_MANAGER
                                             + "?" + EnhydraDMConstants.ACTION + "="
                                             + EnhydraDMConstants.ACTION_LOGOUT);

         Properties prop = ((EnhydraDM) comms.application).getProperties("documentdetail",
                                                                         "documentdetail");
         setI18n(prop, comms.application.getLogChannel());

         if (documentId != null) {
            String mode = EnhydraDMConstants.MODE_DOCUMENT;
            Document document = documentManager.getDocument(documentId);

            String previewerURL = EnhydraDM.previewerURL;
            if (previewerURL == null || previewerURL.equals(""))
               previewerURL = setPreviewerURL(comms);

            page.getElementDocumentPreviewLink().setHref(previewerURL
                                                         + "?fullFilePath="
                                                         + document.getFilepath()
                                                         + "&parseOnFly=true&xsl="
                                                         + EnhydraDM.previewerXSL);
            page.getElementDocumentIdValueLabel().setTextContent(document.getNumber());
            page.getElementDocumentNameValueLabel().setTextContent(document.getName());
            page.getElementFilePathValueLabel().setTextContent(document.getFilepath());
            page.getElementSizeValueLabel()
               .setTextContent(Long.toString(document.getSize()) + " (B)");
            page.getElementMimeTypeValueLabel().setTextContent(document.getMimeType());
            if (document.getLockedBy() != null) {
               page.getElementLockedByValueLabel().setTextContent(document.getLockedBy());
            } else {
               page.getElementLockedByLabel()
                  .getParentNode()
                  .removeChild(page.getElementLockedByLabel());
            }
            if (document.getCheckedOutBy() != null) {
               page.getElementCheckedOutByValueLabel()
                  .setTextContent(document.getCheckedOutBy());
            } else {
               page.getElementCheckedOutByLabel()
                  .getParentNode()
                  .removeChild(page.getElementCheckedOutByLabel());
            }
            if (document.isArchived()) {
               page.getElementArchivrdValue().setAttribute("checked", "true");
            }
            if (document.isAutoversionable()) {
               page.getElementAutovrsionableValue().setAttribute("checked", "true");
            }
            page.getElementCuttentVersionNumberValueLabel()
               .setTextContent(document.getCurrentVersionNumber());
            page.getElementCreatedByValueLabel().setTextContent(document.getCreatedBy());
            Date createdDate = new Date(document.getCreatedDate());
            if (createdDate != null) {
               page.getElementCreatedDateValueLabel()
                  .setTextContent(sdf.format(createdDate));
            }
            page.getElementLastModifiedByValueLabel()
               .setTextContent(document.getLastModifiedBy());
            Date lastModifiedDate = new Date(document.getLastModifiedDate());
            if (lastModifiedDate != null) {
               page.getElementLastModifiedDateValueLabel()
                  .setTextContent(sdf.format(lastModifiedDate));
            }
            if (document.getDeletedBy() != null) {
               page.getElementDeletedByValueLabel()
                  .setTextContent(document.getDeletedBy());
               Date deletedDate = new Date(document.getDeletedDate());
               if (lastModifiedDate != null) {
                  page.getElementDeletedDateValueLabel()
                     .setTextContent(sdf.format(deletedDate));
               }
            } else {
               page.getElementDeletedByLabel()
                  .getParentNode()
                  .removeChild(page.getElementDeletedByLabel());
               page.getElementDeletedDateLabel()
                  .getParentNode()
                  .removeChild(page.getElementDeletedDateLabel());
            }
            if (document.isTemplate()) {
               page.getElementTemplateValue().setAttribute("checked", "true");
               mode = EnhydraDMConstants.MODE_TEMPLATE;
            }
            if (document.getTemplateRef() != null) {
               page.getElementTemplateNameValueLabel()
                  .setTextContent(document.getTemplateRef().getName());
            } else {
               page.getElementTemplateNameLabel()
                  .getParentNode()
                  .removeChild(page.getElementTemplateNameLabel());
            }
            page.getElementBackLink().setHref(EnhydraDMConstants.NAVIG_MANAGER
                                              + "?" + EnhydraDMConstants.MODE + "=" + mode);
         }

         comms.response.writeDOM(page);
      } catch (Throwable e) {
         doCommit = false;
         comms.application.getLogChannel().write(Logger.ERROR, "just failed", e);
         throw new Exception(e);

      } finally {
         if (doCommit) {
            commitTransaction();
         } else {
            rollbackTransaction();
         }
      }
   }

   private String setPreviewerURL(HttpPresentationComms comms) {
      StringBuffer sb = new StringBuffer();
      sb.append(comms.request.getScheme());
      sb.append("://");
      sb.append(comms.request.getServerName());
      sb.append(":");
      sb.append(comms.request.getServerPort());
      sb.append("/");
      sb.append("tdv/Preview.po");
      return sb.toString();
   }

   private void setI18n(Properties prop, LogChannel logChannel) {
      try {
         super.setI18n(prop, page);
      } catch (Exception e) {
         if (null != logChannel) {
            logChannel.write(Logger.WARNING, "documentdetail.properties not found");
         }
      }
   }
}
