/*
 * Enhydra Java Application Server Project The contents of this file are subject to the
 * Enhydra Public License Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on the Enhydra web
 * site ( http://www.enhydra.org/ ). Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific terms governing rights and limitations under the License. The
 * Initial Developer of the Enhydra Application Server is Lutris Technologies, Inc. The
 * Enhydra Application Server and portions created by Lutris Technologies, Inc. are
 * Copyright Lutris Technologies, Inc. All Rights Reserved. Contributor(s): $Id:
 * ErrorHandler.java,v 1.1 2006/09/04 09:07:38 alex Exp $
 */

package org.enhydra.dm.presentation;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.ActionNotAllowedException;
import org.enhydra.dm.business.exceptions.ConfigurationException;
import org.enhydra.dm.business.exceptions.DatabaseException;
import org.enhydra.dm.business.exceptions.DesEncryptionException;
import org.enhydra.dm.business.exceptions.DocumentStoreException;
import org.enhydra.dm.business.exceptions.LockException;

import com.lutris.appserver.server.httpPresentation.*;

/**
 * Trivial error handler.
 */
public class ErrorHandler implements HttpPresentation {
   ErrorHandlerHTML page;

   /**
    * Entry.
    */
   public void run(HttpPresentationComms comms) throws HttpPresentationException {
      page = (ErrorHandlerHTML) comms.xmlcFactory.create(ErrorHandlerHTML.class);

      if (null != comms.exception) {
         if (comms.exception.getCause() instanceof BaseException) {
            BaseException busEx = (BaseException) comms.exception.getCause();
            if (busEx instanceof LockException
                || busEx.getCause() instanceof LockException) {
               setErrorMessageTypeColor("red", LockException.class.getName());
            } else if (busEx instanceof DatabaseException
                       || busEx.getCause() instanceof DatabaseException) {
               setErrorMessageTypeColor("blue", DatabaseException.class.getName());
            } else if (busEx instanceof DocumentStoreException
                       || busEx.getCause() instanceof DocumentStoreException) {
               setErrorMessageTypeColor("green", DocumentStoreException.class.getName());
            } else if (busEx instanceof DesEncryptionException
                       || busEx.getCause() instanceof DesEncryptionException) {
               setErrorMessageTypeColor("orange", DesEncryptionException.class.getName());
            } else if (busEx instanceof ActionNotAllowedException
                       || busEx.getCause() instanceof ActionNotAllowedException) {
               setErrorMessageTypeColor("brown",
                                        ActionNotAllowedException.class.getName());
            } else if (busEx instanceof ConfigurationException
                       || busEx.getCause() instanceof ConfigurationException) {
               setErrorMessageTypeColor("darkblue",
                                        ConfigurationException.class.getName());
            }
         }
         StringWriter stringWriter = new StringWriter();
         comms.exception.printStackTrace();
         comms.exception.printStackTrace(new PrintWriter(stringWriter));

         page.setTextStackTrace(stringWriter.toString());
         String m = comms.exception.getMessage();
         if (null == m)
            m = "NULL message";
         page.setTextErrorMessage(m);
      }
      comms.response.writeDOM(page);
   }

   private void setErrorMessageTypeColor(String color, String type) {
      page.getElementErrorMessageFont().setColor(color);
      page.getElementStackTraceFont().setColor(color);

   }
}
