package org.enhydra.dm.presentation;

import java.io.ByteArrayInputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.FoDocument;
import org.enhydra.dm.api.FoDocumentManager;
import org.enhydra.dm.api.FoDocumentParam;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.util.EnhydraDMConstants;
import org.enhydra.dm.util.TransformUtility;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.PageRedirectException;
import com.lutris.logging.LogChannel;
import com.lutris.logging.Logger;

public class FoEditor extends Base {

   /**
    * Corresponding XMLC object
    */
   private FoEditorHTML page;

   private static final String PROPFILENAME = "foeditor";

   protected FoDocumentManager foDocumentManager = null;

   public void run(HttpPresentationComms comms) throws Exception {
      page = (FoEditorHTML) comms.xmlcFactory.create(FoEditorHTML.class);

      comms.request.getHttpServletRequest()
         .setCharacterEncoding(EnhydraDMConstants.ENCODING);

      String action = comms.request.getParameter(EnhydraDMConstants.ACTION);
      String lang = comms.request.getParameter(EnhydraDMConstants.PARAM_LANG);
      String name = comms.request.getParameter(EnhydraDMConstants.PARAM_NAME);

      if (name != null && !name.equals("")) {
         name = name.endsWith(".pdf") ? name : name + ".pdf";
      }
      String documentId = comms.request.getParameter(EnhydraDMConstants.DOCUMENT_ID);
      String templateId = comms.request.getParameter(EnhydraDMConstants.TEMPLATE_ID);

      documentManager = ((EnhydraDM) comms.application).getDocumentManager();
      documentStore = ((EnhydraDM) comms.application).getDocumentStore();
      foDocumentManager = ((EnhydraDM) comms.application).getFoDocumentManager();

      boolean doCommit = true;
      try {
         initTransaction();
         String user = getUser(comms);

         page.setTextUser(user);

         page.getElementLogoutLink().setHref(EnhydraDMConstants.NAVIG_MANAGER
                                             + "?" + EnhydraDMConstants.ACTION + "="
                                             + EnhydraDMConstants.ACTION_LOGOUT);

         page.getElementName().setValue(name);
         page.getElementNameLabelValue().setTextContent(name);

         Enumeration paramNames = comms.request.getParameterNames();

         // Store editor content, "Document properties" and "PDF properties", 
         // embed editor content and "Document properties" and create pdf file, 
         // on the end store new pdf file or new pdf file version. 
         if (EnhydraDMConstants.ACTION_SAVE.equals(action)) {
            // Create new or update existing PDF document
            try {
               HashMap foParams = new HashMap();
               HashMap pdfParams = new HashMap();

               String paramName = null;
               String paramValue = null;

               while (paramNames.hasMoreElements()) {
                  paramName = paramNames.nextElement().toString();
                  paramValue = comms.request.getParameter(paramName);
                  if (paramName.startsWith(EnhydraDMConstants.FO_PARAM_PREFIX)) {
                     foParams.put(paramName, paramValue);
                  } else if (paramName.startsWith(EnhydraDMConstants.PDF_PARAM_PREFIX)) {
                     if (!paramName.endsWith(EnhydraDMConstants.PDF_PARAM_USERPASSWORD)
                         && !paramName.endsWith(EnhydraDMConstants.PDF_PARAM_OWNERPASSWORD)) {
                        pdfParams.put(paramName, new Boolean(paramValue));
                     } else {
                        pdfParams.put(paramName, paramValue);
                     }
                  }
               }

               // html2xhtml for TinyMCE submission
               Iterator entries = foParams.entrySet().iterator();
               while (entries.hasNext()) {
                  Map.Entry me = (Map.Entry) entries.next();
                  paramName = (String) me.getKey();

                  if (paramName.endsWith(EnhydraDMConstants.NODE_PARAM_SUFIX)) {
                     String content = (String) me.getValue();
                     content = TransformUtility.html2xhtml(content);
                     foParams.put(paramName, content);
                  }
               }

               // Save fo document and its parameters for futer edit option
               String foDocumentId = this.createFoDocument(foParams, pdfParams);

               // xhtml2fo for transformed TinyMCE submission
               entries = foParams.entrySet().iterator();
               while (entries.hasNext()) {
                  Map.Entry me = (Map.Entry) entries.next();
                  paramName = (String) me.getKey();

                  if (paramName.endsWith(EnhydraDMConstants.NODE_PARAM_SUFIX)) {
                     String content = (String) me.getValue();
                     content = TransformUtility.xhtml2fo(content);
                     foParams.put(paramName, content);
                  }
               }

               // prepare XSLFast transformation input
               byte[] cont = TransformUtility.params2xml(foParams)
                  .getBytes(EnhydraDMConstants.ENCODING);

               // Setting optional PDF attributes
               pdfParams.put("producer", "Together Serbia");
               pdfParams.put("creator", "Enhydra Document Management Application");
               pdfParams.put("author", "Enhydra Development Team");
               pdfParams.put("title", "Demonstration Document");
               pdfParams.put("keywords", "");
               pdfParams.put("resolution", new Integer(300));
               pdfParams.put("date", new Date());

               org.enhydra.dm.api.Document template = documentManager.getDocument(templateId);

               // create PDF from input
               cont = TransformUtility.xml2pdf(cont, pdfParams, template.getFilepath());

               String mimeType = "application/pdf";
               if (documentId != null && !documentId.equals("")) {
                  // create new version of PDF document
                  this.updateDocument(documentId, name, cont, user, foDocumentId);
               } else {
                  // create new PDF document
                  this.createDocument(name,
                                      cont,
                                      mimeType,
                                      user,
                                      foDocumentId,
                                      templateId);
               }
               throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_MANAGER
                                                     + "?" + EnhydraDMConstants.MODE
                                                     + "="
                                                     + EnhydraDMConstants.MODE_DOCUMENT);
            } catch (BaseException e) {
               throw new Exception(e);
            }
         } else {
            Properties langProp = ((EnhydraDM) comms.application).getProperties(PROPFILENAME
                                                                                      + "_"
                                                                                      + lang,
                                                                                PROPFILENAME);
            setI18n(langProp, comms.application.getLogChannel(), lang);

            if (EnhydraDMConstants.ACTION_NEW.equals(action)) {
               // Open new
               page.getElementTemplateId().setValue(templateId);
               comms.response.writeDOM(page);
            } else if (EnhydraDMConstants.ACTION_EDIT.equals(action)) {
               // Edit existing
               org.enhydra.dm.api.Document document = documentManager.getDocument(documentId);
               String foDocumentId = document.getFoDocumentRef().getId();
               templateId = document.getTemplateRef().getNumber();

               page.getElementDocumentId().setValue(documentId);
               page.getElementTemplateId().setValue(templateId);

               // lock document
               if (documentId != null) {
                  page.getElementBackLink().setHref(EnhydraDMConstants.NAVIG_FOEDITOR
                                                    + "?" + EnhydraDMConstants.ACTION
                                                    + "="
                                                    + EnhydraDMConstants.ACTION_UNLOCK
                                                    + "&"
                                                    + EnhydraDMConstants.DOCUMENT_ID
                                                    + "=" + documentId);
                  this.lockDocument(documentId, user);
               }

               FoDocument foDocument = foDocumentManager.getFoDocumentById(foDocumentId);
               FoDocumentParam[] foDocumentParams = foDocumentManager.getFoDocumentParametersById(foDocument.getId());
               
               // load document content
               for (int i = 0; i < foDocumentParams.length; i++) {
                  String paramName = foDocumentParams[i].getName();
                  String patramValue = foDocumentParams[i].getContent();

                  if (paramName.endsWith(EnhydraDMConstants.NODE_PARAM_SUFIX)) {
                     DocumentBuilder db = DocumentBuilderFactory.newInstance()
                        .newDocumentBuilder();
                     byte[] b = patramValue.getBytes(EnhydraDMConstants.ENCODING);
                     Document documentNode = db.parse(new ByteArrayInputStream(b));

                     if (documentNode != null) {
                        Node bodyNode = documentNode.getElementsByTagName("body").item(0);
                        NodeList bodyChildren = bodyNode.getChildNodes();

                        for (int j = 0; j < bodyChildren.getLength(); j++) {
                           Node contentNode = bodyChildren.item(j);
                           if (contentNode != null) {
                              contentNode = page.importNode(contentNode, true);
                              page.getElementById(paramName).appendChild(contentNode);
                           }
                        }
                     }
                  } else {
                     page.getElementById(paramName).setAttribute("value", patramValue);
                  }
               }

               // load pdf properties
               page.getElementById(EnhydraDMConstants.PDF_PARAM_PREFIX
                                   + EnhydraDMConstants.PDF_PARAM_USERPASSWORD)
                  .setAttribute("value", foDocument.getUserPassword());
               page.getElementById(EnhydraDMConstants.PDF_PARAM_PREFIX
                                   + EnhydraDMConstants.PDF_PARAM_OWNERPASSWORD)
                  .setAttribute("value", foDocument.getOwnerPassword());

               if (foDocument.isAllowPrint())
                  page.getElementById(EnhydraDMConstants.PDF_PARAM_PREFIX
                                      + EnhydraDMConstants.PDF_PARAM_ALLOWPRINT)
                     .setAttribute("checked", "true");

               if (foDocument.isAllowCopyContent())
                  page.getElementById(EnhydraDMConstants.PDF_PARAM_PREFIX
                                      + EnhydraDMConstants.PDF_PARAM_ALLOWCOPYCONTENT)
                     .setAttribute("checked", "true");

               if (foDocument.isAllowEditContent())
                  page.getElementById(EnhydraDMConstants.PDF_PARAM_PREFIX
                                      + EnhydraDMConstants.PDF_PARAM_ALLOWEDITCONTENT)
                     .setAttribute("checked", "true");

               if (foDocument.isAllowEditAnnotation())
                  page.getElementById(EnhydraDMConstants.PDF_PARAM_PREFIX
                                      + EnhydraDMConstants.PDF_PARAM_ALLOWEDITANNOTATION)
                     .setAttribute("checked", "true");

               comms.response.writeDOM(page);
            } else if (EnhydraDMConstants.ACTION_UNLOCK.equals(action)) {
               try {
                  this.unlockDocument(documentId, user);

                  throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_MANAGER
                                                        + "?"
                                                        + EnhydraDMConstants.MODE
                                                        + "="
                                                        + EnhydraDMConstants.MODE_DOCUMENT);
               } catch (BaseException e) {
                  throw new Exception(e);
               }
            }
         }
      } catch (PageRedirectException e) {
         throw e;
      } catch (Throwable e) {
         doCommit = false;
         comms.application.getLogChannel().write(Logger.ERROR, "just failed", e);
         throw new Exception(e);

      } finally {
         if (doCommit) {
            commitTransaction();
         } else {
            rollbackTransaction();
         }
      }
   }

   /**
    * Method enable mulilanguage on FoEditor page. It recives propertie file with
    * translated label names and sets them on page. If ptoperties file is null, label
    * names will be in English.
    * 
    * @param langProp
    * @param logChannel
    * @param lang
    */
   private void setI18n(Properties langProp, LogChannel logChannel, String lang) {
      try {
         super.setI18n(langProp, page);

         String language = langProp.getProperty("language");
         if (language != null) {
            page.getElementLang().setTextContent("var langId = \"" + language + "\";");
         }
      } catch (Exception e) {
         if (null != logChannel) {
            lang = lang == null ? PROPFILENAME : PROPFILENAME + "_" + lang;
            logChannel.write(Logger.WARNING, lang + ".properties not found");
         }
      }
   }
   
// ------ Document Management Methods
   
   /**
    * Method store pdf file created from FoEditor content.
    * 
    * @param documentName
    * @param documentContent
    * @param mimeType
    * @param user
    * @param foDocumentId
    * @param templateId
    * @throws BaseException
    */
   private void createDocument(String documentName,
                               byte[] documentContent,
                               String mimeType,
                               String user,
                               String foDocumentId,
                               String templateId) throws BaseException {
      String documentPath = documentStore.saveDocumentContent(documentName,
                                                              documentContent);
      documentManager.create(documentName,
                             documentPath,
                             mimeType,
                             user,
                             foDocumentId,
                             false,
                             templateId);
   }

   /**
    * Method update pdf file created and changed using FoEditor.
    * 
    * @param documentId
    * @param documentName
    * @param documentContent
    * @param user
    * @param foDocumentId
    * @throws BaseException
    */
   private void updateDocument(String documentId,
                               String documentName,
                               byte[] documentContent,
                               String user,
                               String foDocumentId) throws BaseException {
      String documentPath = documentStore.saveDocumentContent(documentName,
                                                              documentContent);
      documentManager.update(documentId, documentPath, user, foDocumentId);
   }

   /**
    * Method lock pdf file created from FoEditor content and edited in FoEditor.
    * 
    * @param documentId
    * @param user
    * @throws BaseException
    */
   private void lockDocument(String documentId, String user) throws BaseException {
      documentManager.lock(documentId, user);
   }

   /**
    * Method unlock pdf file edited in FoEditor
    * 
    * @param documentId
    * @param user
    * @throws BaseException
    */
   private void unlockDocument(String documentId, String user) throws BaseException {
      documentManager.unlock(documentId, user);
   }

   /**
    * Metod store editor content.
    * 
    * @param foParams
    * @param pdfParams
    * @return
    * @throws BaseException
    */
   private String createFoDocument(HashMap foParams, HashMap pdfParams)
      throws BaseException {

      return foDocumentManager.create(foParams,
                                      (String) pdfParams.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                             + EnhydraDMConstants.PDF_PARAM_USERPASSWORD),
                                      (String) pdfParams.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                             + EnhydraDMConstants.PDF_PARAM_OWNERPASSWORD),
                                      ((Boolean) pdfParams.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                               + EnhydraDMConstants.PDF_PARAM_ALLOWPRINT)).booleanValue(),
                                      ((Boolean) pdfParams.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                               + EnhydraDMConstants.PDF_PARAM_ALLOWCOPYCONTENT)).booleanValue(),
                                      ((Boolean) pdfParams.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                               + EnhydraDMConstants.PDF_PARAM_ALLOWEDITCONTENT)).booleanValue(),
                                      ((Boolean) pdfParams.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                               + EnhydraDMConstants.PDF_PARAM_ALLOWEDITANNOTATION)).booleanValue());
   }
}
