/*
 * Enhydra super-servlet presentation object
 */

package org.enhydra.dm.presentation;

// Enhydra SuperServlet imports
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Properties;

import javax.activation.MimetypesFileTypeMap;
import javax.servlet.ServletContext;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.FoDocument;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.ActionNotAllowedException;
import org.enhydra.dm.business.exceptions.FileSizeException;
import org.enhydra.dm.util.EnhydraDMConstants;
import org.enhydra.dm.util.MimeUtility;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLAnchorElement;
import org.w3c.dom.html.HTMLImageElement;
import org.w3c.dom.html.HTMLInputElement;
import org.w3c.dom.html.HTMLOptionElement;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;
import com.lutris.appserver.server.httpPresentation.HttpPresentationResponse;
import com.lutris.logging.Logger;
import com.lutris.mime.ContentHeader;
import com.lutris.mime.MimeHeader;
import com.lutris.mime.MultipartMimeInput;
import com.lutris.mime.MultipartMimeInputStream;

public class Manager extends Base {

   /**
    * Corresponding XMLC object
    */
   private ManagerHTML page;

   // --------------------------------------------- Presentation Object Methods

   public void run(HttpPresentationComms comms) throws Exception {

      Properties editableMT = ((EnhydraDM) comms.application).getProperties(EnhydraDMConstants.PROP_EDITABLE_MT_FILE_NAME,
                                                                            EnhydraDMConstants.PROP_EDITABLE_MT_FILE_NAME);

      page = (ManagerHTML) comms.xmlcFactory.create(ManagerHTML.class);

      comms.request.getHttpServletRequest()
         .setCharacterEncoding(EnhydraDMConstants.ENCODING);

      String action = comms.request.getParameter(EnhydraDMConstants.ACTION);
      String documentId = comms.request.getParameter(EnhydraDMConstants.DOCUMENT_ID);
      String mode = comms.request.getParameter(EnhydraDMConstants.MODE);
      String templateId = comms.request.getParameter(EnhydraDMConstants.TEMPLATE_ID);
      String newDocumentName = comms.request.getParameter(EnhydraDMConstants.PARAM_NAME);

      boolean isTemplate = true;
      if (!EnhydraDMConstants.MODE_TEMPLATE.equals(mode)) {
         mode = EnhydraDMConstants.MODE_DOCUMENT;
         isTemplate = false;
      }

      documentManager = ((EnhydraDM) comms.application).getDocumentManager();
      documentStore = ((EnhydraDM) comms.application).getDocumentStore();

      boolean doCommit = true;
      try {
         initTransaction();
         String user = getUser(comms);

         if (action != null && !"".equals(action)) {

            if (action.equals(EnhydraDMConstants.ACTION_DELETE)) {
               deleteDocument(comms, documentId, user);
            } else if (action.equals(EnhydraDMConstants.ACTION_CHECKOUT)) {
               checkoutDocument(comms, documentId, user);
            } else if (action.equals(EnhydraDMConstants.ACTION_UNCHECKOUT)) {
               unCheckoutDocument(comms, documentId, user);
            } else if (action.equals(EnhydraDMConstants.ACTION_SWITCHVERSIONABLE)) {
               switchVersionable(comms, documentId, user);
            } else if (action.equals(EnhydraDMConstants.ACTION_CREATEVERSION)) {
               createVersion(comms, documentId, user);
            } else if (action.equals(EnhydraDMConstants.ACTION_SWITCHARCHIVED)) {
               switchArchived(comms, documentId, user);
            } else if (EnhydraDMConstants.ACTION_DOWNLOAD.equals(action)) {
               downloadVersionFile(comms, documentId);
               throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_VERSIONS
                                                     + "?"
                                                     + EnhydraDMConstants.DOCUMENT_ID
                                                     + "=" + documentId + "&"
                                                     + EnhydraDMConstants.MODE + "="
                                                     + mode);
            } else if (action.equals(EnhydraDMConstants.ACTION_DUPLICATEDOCUMENT)) {
               duplicateDocument(comms, documentId, user);
            } else if (action.equals(EnhydraDMConstants.ACTION_LOGOUT)) {
               comms.request.getHttpServletRequest().getSession().invalidate();
            } else if (action.equals(EnhydraDMConstants.ACTION_UPLOAD)) {
               String header = comms.request.getHeader(EnhydraDMConstants.HEAD_CONTENT_TYPE);
               String fileSize = comms.request.getHeader(EnhydraDMConstants.HEAD_CONTENT_LENGTH);
               int fileSizeInt = (new Integer(fileSize)).intValue();
               if (fileSizeInt > EnhydraDM.allowedFileSizeForUpload) {
                  throw new FileSizeException("File you are trying to upload is too big!");
               }
               if (header != null && !"".equals(header)) {
                  try {
                     ContentHeader contentHeader = new ContentHeader(header);
                     MultipartMimeInput mInput = new MultipartMimeInput(comms.request.getInputStream(),
                                                                        contentHeader);
                     MultipartMimeInputStream mime = mInput.nextPart(EnhydraDMConstants.ENCODING);
                     String documentName = null;
                     byte[] documentContent = null;
                     while (mime != null) {
                        MimeHeader mh = mime.getHeader(EnhydraDMConstants.HEAD_CONTENT_DISPOSITION);
                        String paramName = mh.getParameter(EnhydraDMConstants.PARAM_NAME);
                        if (EnhydraDMConstants.PARAM_UPLOAD_DOCUMENT.equals(paramName)) {
                           documentName = mh.getParameter(EnhydraDMConstants.PARAM_FILE_NAME);
                           // set the corresponding (not absolute path) file name (depends
                           // of
                           // the browser)
                           File tmpfile = new File(documentName);
                           documentName = tmpfile.getName();
                           documentContent = getBinaryContent(mime);
                        } else if (EnhydraDMConstants.ACTION.equals(paramName)) {
                           action = getString(mime);
                        } else if (EnhydraDMConstants.DOCUMENT_ID.equals(paramName)) {
                           documentId = getString(mime);
                        } else if (EnhydraDMConstants.MODE.equals(paramName)) {
                           mode = getString(mime);
                           isTemplate = true;
                           if (!EnhydraDMConstants.MODE_TEMPLATE.equals(mode)) {
                              mode = EnhydraDMConstants.MODE_DOCUMENT;
                              isTemplate = false;
                           }
                        }
                        mime = mInput.nextPart();
                     }
                     if (documentName != null && documentContent != null) {
                        if (EnhydraDMConstants.ACTION_CHECKIN.equals(action)) {
                           checkinDocument(documentId,
                                           documentName,
                                           documentContent,
                                           user);
                        } else {

                           String mimeType = MimeUtility.mimeTypeMapper(editableMT,
                                                                        documentName);

                           if (null == mimeType) {
                              ServletContext servletContext = comms.application.getHttpPresentationManager()
                                 .getServletContext();
                              mimeType = servletContext != null ? servletContext.getMimeType(documentName)
                                                               : "application/octet-stream";
                           }
                           mimeType = mimeType != null ? mimeType
                                                      : "application/octet-stream";

                           create(documentName,
                                  documentContent,
                                  mimeType,
                                  user,
                                  null,
                                  isTemplate,
                                  null);
                        }
                     }
                  } catch (BaseException e) {
                     if (null != comms.application.getLogChannel()) {
                        comms.application.getLogChannel()
                           .write(Logger.ERROR,
                                  "Action :"
                                        + action + " failure! Reason: " + e.getMessage());
                     }
                     throw new Exception(e);
                  }
               }
            } else if (EnhydraDMConstants.ACTION_CREATE.equals(action)) {
               Document template = documentManager.getDocument(templateId);

               String filepath = template.getFilepath();
               String extension = filepath.substring(filepath.lastIndexOf("."));

               if (!newDocumentName.endsWith(extension)) {
                  newDocumentName += extension;
               }

               byte[] documentContent = documentStore.loadDocumentContent(template.getFilepath());
               create(newDocumentName,
                      documentContent,
                      template.getMimeType(),
                      user,
                      null,
                      isTemplate,
                      templateId);
            } else if (EnhydraDMConstants.ACTION_MASS_DELETE.equals(action)) {
               massDeleteDocument(comms, user);
            }
            throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_MANAGER
                                                  + "?" + EnhydraDMConstants.MODE + "="
                                                  + mode);
         }
         page.setTextUser("User: " + user);
         page.getElementAllowedExtensions()
            .setValue(EnhydraDM.allowedExtensionsForUpload);
         fillFilesetTable(comms, user, isTemplate, editableMT);
         comms.response.writeDOM(page);
      } catch (BaseException ex) {
         doCommit = false;
         if (null != comms.application.getLogChannel()) {
            comms.application.getLogChannel().write(Logger.ERROR,
                                                    "Action :"
                                                          + action + " failure! Reason: "
                                                          + ex.getMessage());
         }
         throw new Exception(ex);
      } finally {
         if (doCommit) {
            commitTransaction();
         } else {
            rollbackTransaction();
         }
      }
   }

   private void fillFilesetTable(HttpPresentationComms comms,
                                 String user,
                                 boolean isTemplate,
                                 Properties editableMT) throws BaseException {
      String mode = EnhydraDMConstants.MODE_DOCUMENT;
      if (isTemplate) {
         mode = EnhydraDMConstants.MODE_TEMPLATE;
      }

      Node fileRow = page.getElementDocumentRow();
      Document[] documents = null;
      documents = documentManager.getDocuments(isTemplate);
      SimpleDateFormat sdf = new SimpleDateFormat(EnhydraDMConstants.DATE_FORMAT);

      HTMLAnchorElement readLink = page.getElementReadLink();
      HTMLAnchorElement editLink = page.getElementEditLink();
      HTMLAnchorElement logoutLink = page.getElementLogoutLink();
      HTMLAnchorElement recycleBinLink = page.getElementRecycleBinLink();
      HTMLAnchorElement swichModeLink = page.getElementSwichModeLink();
      HTMLAnchorElement documentDetailLink = page.getElementDocumentDetailLink();
      HTMLAnchorElement checkoutLink = page.getElementCheckoutLink();
      HTMLAnchorElement unCheckoutLink = page.getElementUnCheckoutLink();
      HTMLAnchorElement checkinLink = page.getElementCheckinLink();
      HTMLAnchorElement deleteLink = page.getElementDeleteLink();
      HTMLAnchorElement versionsLink = page.getElementVersionsLink();
      HTMLAnchorElement createVersLink = page.getElementCreateVersionLink();
      HTMLAnchorElement swithcVersionableLink = page.getElementSwithcVersionableLink();
      HTMLAnchorElement archivedLink = page.getElementArchivedLink();
      HTMLAnchorElement c2cLink = page.getElementC2cLink();
      HTMLAnchorElement saaLink = page.getElementSaaLink();
      HTMLAnchorElement sabLink = page.getElementSabLink();
      HTMLAnchorElement duplicateLink = page.getElementDuplicateLink();
      HTMLAnchorElement downloadLink = page.getElementDownloadLink();

      HTMLInputElement selectCheckbox = page.getElementSelectDocCheckbox();

      HTMLImageElement lockImage = page.getElementLockImage();
      HTMLImageElement versionableImage = page.getElementVersionableImage();
      HTMLImageElement isArchivedImage = page.getElementIsArchivedImage();
      HTMLImageElement editImage = page.getElementEditImage();
      HTMLImageElement checkoutImage = page.getElementCheckoutImage();
      HTMLImageElement unCheckoutImage = page.getElementUnCheckoutImage();
      HTMLImageElement checkinImage = page.getElementCheckinImage();
      HTMLImageElement deleteImage = page.getElementDeleteImage();
      HTMLImageElement versionsImage = page.getElementVersionsImage();
      HTMLImageElement createVersImage = page.getElementCreateVersionImage();
      HTMLImageElement swithcVersionableImage = page.getElementSwitchVersionableImage();
      HTMLImageElement archivedImage = page.getElementArchivedImage();

      logoutLink.setHref(EnhydraDMConstants.NAVIG_MANAGER
                         + "?" + EnhydraDMConstants.ACTION + "="
                         + EnhydraDMConstants.ACTION_LOGOUT);

      if (!isTemplate) {
         page.setTextSwichMode("Templates");
         page.setTextListHeader("Document List");
         swichModeLink.setHref(EnhydraDMConstants.NAVIG_MANAGER
                               + "?" + EnhydraDMConstants.MODE + "="
                               + EnhydraDMConstants.MODE_TEMPLATE);
      } else {
         page.setTextSwichMode("Documents");
         page.setTextListHeader("Template List");
         swichModeLink.setHref(EnhydraDMConstants.NAVIG_MANAGER
                               + "?" + EnhydraDMConstants.MODE + "="
                               + EnhydraDMConstants.MODE_DOCUMENT);
      }

      recycleBinLink.setHref(EnhydraDMConstants.NAVIG_RECYCLEBIN
                             + "?" + EnhydraDMConstants.MODE + "=" + mode);

      page.getElementContentMode().setValue(mode);
      page.getElementCreateMode().setValue(mode);

      if (documents != null) {

         for (int i = 0; i < documents.length; i++) {
            String documentName = documents[i].getName();
            String documentId = documents[i].getNumber().toString();
            long documentSize = documents[i].getSize();
            FoDocument foDocumentRef = documents[i].getFoDocumentRef();

            boolean documentLoked = documents[i].getLockedBy() != null
                                    || documents[i].getCheckedOutBy() != null;

            documentDetailLink.setHref(EnhydraDMConstants.NAVIG_DETAILS
                                       + "?" + EnhydraDMConstants.DOCUMENT_ID + "="
                                       + documentId + "&" + EnhydraDMConstants.MODE + "="
                                       + mode);
            page.setTextDocumentName(documentName);
            page.setTextDocumentId(documentId);
            page.setTextDocumentSize(String.valueOf(documentSize));

            Date mainDate = new Date(documents[i].getLastModifiedDate());
            if (mainDate != null) {
               String strMainDate = sdf.format(mainDate);
               page.setTextDocumentLastModifiedDate(strMainDate);
            } else {
               page.setTextDocumentLastModifiedDate("-");
            }
            if (documents[i].isArchived()) {
               isArchivedImage.setSrc("media/isArchived.gif");
               lockImage.setSrc("media/blank.gif");
               versionableImage.setSrc("media/blank.gif");
            } else {
               isArchivedImage.setSrc("media/blank.gif");
               if (documentLoked) {
                  lockImage.setSrc("media/lock.gif");
               } else {

                  lockImage.setSrc("media/unlock.gif");

               }
               if (documents[i].isAutoversionable()) {
                  versionableImage.setSrc("media/gear_ok.gif");
               } else {

                  versionableImage.setSrc("media/gear_forbidden.gif");

               }
            }

            String parameters = null;

            parameters = EnhydraDMConstants.ACTION
                         + "=" + EnhydraDMConstants.ACTION_READ + "&"
                         + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId;

            if (EnhydraDM.desenc != null) {
               parameters = EnhydraDM.desenc.encrypt(parameters);
               parameters = EnhydraDMConstants.PARAM + "=" + parameters;
            }

            try {
               parameters = URLEncoder.encode(parameters, EnhydraDMConstants.ENCODING);
            } catch (Exception ex) {

            }

            readLink.setHref(EnhydraDMConstants.NAVIG_WEBDAV
                             + "/" + documents[i].getName() + "?" + parameters);

            c2cLink.setHref("javascript: copy2clipboard('"
                            + EnhydraDMConstants.NAVIG_WEBDAV + "/"
                            + documents[i].getName() + "?" + parameters + "');");
            saaLink.setHref("javascript: sendasattachement('"
                            + EnhydraDMConstants.NAVIG_WEBDAV + "/"
                            + documents[i].getName() + "?" + parameters + "');");

            sabLink.setHref("javascript: sendShortcut('"
                            + EnhydraDMConstants.NAVIG_WEBDAV + "/"
                            + documents[i].getName() + "?" + parameters + "');");

            if (!documents[i].isArchived()
                && !documentLoked
                && (MimeUtility.isEditableDocument(editableMT, documents[i].getMimeType()) || foDocumentRef != null)) {
               if (foDocumentRef != null) {
                  parameters = EnhydraDMConstants.ACTION
                               + "=" + EnhydraDMConstants.ACTION_EDIT + "&"
                               + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                               + EnhydraDMConstants.PARAM_NAME + "="
                               + documents[i].getName();
                  editLink.setHref(EnhydraDMConstants.NAVIG_FOEDITOR + "?" + parameters);
               } else {
                  parameters = EnhydraDMConstants.ACTION
                               + "=" + EnhydraDMConstants.ACTION_EDIT + "&"
                               + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                               + EnhydraDMConstants.PARAM_NAME + "="
                               + documents[i].getName();
                  if (EnhydraDM.desenc != null) {
                     parameters = EnhydraDM.desenc.encrypt(parameters);
                     parameters = EnhydraDMConstants.PARAM + "=" + parameters;
                  }

                  try {
                     parameters = URLEncoder.encode(parameters,
                                                    EnhydraDMConstants.ENCODING);
                  } catch (Exception ex) {

                  }

                  editLink.setHref(EnhydraDMConstants.NAVIG_WEBDAV
                                   + "/" + documents[i].getName() + "?" + parameters);

               }
               editImage.setSrc("media/edit.gif");
            } else {
               editLink.removeAttribute("href");
               editImage.setSrc("media/editDis.gif");
            }

            if (!documentLoked && !documents[i].isArchived()) {

               String value = "javascript:checkoutDocument('"
                              + documentId + "','" + mode + "')";

               checkoutLink.setAttribute("href", value);

               checkoutImage.setSrc("media/checkout.gif");
            } else {
               checkoutLink.removeAttribute("href");

               checkoutImage.setSrc("media/checkoutDis.gif");
            }

            if (!documentLoked
                && !documents[i].isArchived() && !documents[i].isAutoversionable()) {

               String value = EnhydraDMConstants.NAVIG_MANAGER
                              + "?" + EnhydraDMConstants.ACTION + "="
                              + EnhydraDMConstants.ACTION_CREATEVERSION + "&"
                              + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                              + EnhydraDMConstants.MODE + "=" + mode;

               createVersLink.setAttribute("href", value);

               createVersImage.setSrc("media/createVers.gif");
            } else {
               createVersLink.removeAttribute("href");

               createVersImage.setSrc("media/createVersDis.gif");
            }

            String appUser = documents[i].getCheckedOutBy();
            if (appUser != null && appUser.equals(user)) {

               String value = EnhydraDMConstants.NAVIG_MANAGER
                              + "?" + EnhydraDMConstants.ACTION + "="
                              + EnhydraDMConstants.ACTION_UNCHECKOUT + "&"
                              + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                              + EnhydraDMConstants.MODE + "=" + mode;
               unCheckoutLink.setAttribute("href", value);

               unCheckoutImage.setSrc("media/uncheckout.gif");
            } else {
               unCheckoutLink.removeAttribute("href");

               unCheckoutImage.setSrc("media/uncheckoutDis.gif");
            }
            if (appUser != null && appUser.equals(user)) {

               String value = EnhydraDMConstants.NAVIG_UPLOAD
                              + "?" + EnhydraDMConstants.ACTION + "="
                              + EnhydraDMConstants.ACTION_CHECKIN + "&"
                              + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                              + EnhydraDMConstants.MODE + "=" + mode;
               checkinLink.setAttribute("href", value);

               checkinImage.setSrc("media/checkin.gif");
            } else {
               checkinLink.removeAttribute("href");
               checkinImage.setSrc("media/checkinDis.gif");
            }
            if (documentLoked || documents[i].isArchived()) {
               deleteLink.removeAttribute("href");
               deleteImage.setSrc("media/deleteDis.gif");
            } else {

               String value = EnhydraDMConstants.NAVIG_MANAGER
                              + "?" + EnhydraDMConstants.ACTION + "="
                              + EnhydraDMConstants.ACTION_DELETE + "&"
                              + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                              + EnhydraDMConstants.MODE + "=" + mode;
               deleteLink.setAttribute("href", value);

               deleteImage.setSrc("media/delete.gif");
            }

            if (documents[i].isArchived()) {
               versionsLink.removeAttribute("href");

               versionsImage.setSrc("media/versionsDis.gif");

            } else {

               String value = EnhydraDMConstants.NAVIG_VERSIONS
                              + "?" + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId
                              + "&" + EnhydraDMConstants.MODE + "=" + mode;
               versionsLink.setAttribute("href", value);

               versionsImage.setSrc("media/versions.gif");

            }

            if (documentLoked || documents[i].isArchived()) {
               swithcVersionableLink.removeAttribute("href");
               swithcVersionableImage.setSrc("media/historyDis.gif");
            } else {

               String value = EnhydraDMConstants.NAVIG_MANAGER
                              + "?" + EnhydraDMConstants.ACTION + "="
                              + EnhydraDMConstants.ACTION_SWITCHVERSIONABLE + "&"
                              + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                              + EnhydraDMConstants.MODE + "=" + mode;
               swithcVersionableLink.setAttribute("href", value);

               swithcVersionableImage.setSrc("media/history.gif");
            }
            if (documentLoked) {
               archivedLink.removeAttribute("href");
               archivedImage.setSrc("media/archivedDis.gif");
            } else {

               String value = EnhydraDMConstants.NAVIG_MANAGER
                              + "?" + EnhydraDMConstants.ACTION + "="
                              + EnhydraDMConstants.ACTION_SWITCHARCHIVED + "&"
                              + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId + "&"
                              + EnhydraDMConstants.MODE + "=" + mode;
               archivedLink.setAttribute("href", value);

               archivedImage.setSrc("media/archived.gif");
            }

            selectCheckbox.setName(EnhydraDMConstants.HTML_NAME_SELECT_DOC_CHECKBOX
                                   + documentId);
            if (!documentLoked && !documents[i].isArchived()) {
               selectCheckbox.setValue(documentId);
            } else {
               selectCheckbox.setValue("");
            }

            duplicateLink.setAttribute("href",
                                       EnhydraDMConstants.NAVIG_MANAGER
                                             + "?"
                                             + EnhydraDMConstants.ACTION
                                             + "="
                                             + EnhydraDMConstants.ACTION_DUPLICATEDOCUMENT
                                             + "&" + EnhydraDMConstants.DOCUMENT_ID + "="
                                             + documentId + "&" + EnhydraDMConstants.MODE
                                             + "=" + mode);

            downloadLink.setAttribute("href", EnhydraDMConstants.NAVIG_MANAGER
                                              + "?" + EnhydraDMConstants.ACTION + "="
                                              + EnhydraDMConstants.ACTION_DOWNLOAD + "&"
                                              + EnhydraDMConstants.DOCUMENT_ID + "="
                                              + documentId + "&"
                                              + EnhydraDMConstants.MODE + "=" + mode);

            fileRow.getParentNode().insertBefore(fileRow.cloneNode(true), fileRow);
         }
      }
      fileRow.getParentNode().removeChild(fileRow);

      // create from template
      HTMLOptionElement templatesComboRow = page.getElementTemplatesOption();
      documents = documentManager.getDocuments(true);

      if (isTemplate) {
         page.getElementNewPdf().setAttribute("class", "hidden");
      }

      page.setTextTemplateName("Please select");
      templatesComboRow.setValue("-1");
      templatesComboRow.getParentNode().insertBefore(templatesComboRow.cloneNode(true),
                                                     templatesComboRow);

      if (documents != null) {
         for (int i = 0; i < documents.length; i++) {
            String templateName = documents[i].getName();
            String templateOid = documents[i].getNumber().toString();

            if (!isTemplate || !documents[i].getMimeType().equals("application/xml")) {
               page.setTextTemplateName(templateName);
               templatesComboRow.setValue(templateOid);
               templatesComboRow.getParentNode()
                  .insertBefore(templatesComboRow.cloneNode(true), templatesComboRow);
            }
         }
      }

      templatesComboRow.getParentNode().removeChild(templatesComboRow);
   }

   // --------------------------------------------- Transaction handling Methods

   // --------------------------------------------- Multipart parameter handling methods

   /**
    * Method returns string parameter value
    * 
    * @param part
    * @return parameter value
    * @throws IOException
    */
   private String getString(MultipartMimeInputStream part) throws IOException {
      byte[] buf = new byte[1024];
      StringBuffer res = new StringBuffer(1024);
      int b = 0;
      while ((b = part.read(buf, 0, buf.length)) != -1) {
         res.append(new String(buf, 0, b, EnhydraDMConstants.ENCODING));
      }
      return res.toString();
   }

   /**
    * Method retrieves binary parameter value
    * 
    * @param mime
    * @return binry content
    * @throws Exception
    * @throws BaseException
    */
   private byte[] getBinaryContent(MultipartMimeInputStream mime) throws Exception {
      byte[] buf = new byte[1024];
      ByteArrayOutputStream baos = null;
      BufferedOutputStream bos = null;
      try {
         baos = new ByteArrayOutputStream();
         bos = new BufferedOutputStream(baos);

         int b = 0;
         while ((b = mime.read(buf, 0, buf.length)) != -1) {
            bos.write(buf, 0, b);
         }
         bos.flush();

         return baos.toByteArray();
      } catch (Exception ex) {
         throw ex;
      } finally {
         if (baos != null)
            try {
               baos.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
         if (bos != null)
            try {
               bos.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
      }

   }

   // ------ Document Management Methods

   private void checkinDocument(String documentId,
                                String documentName,
                                byte[] documentContent,
                                String user) throws BaseException {

      String documentPath = documentStore.saveDocumentContent(documentName,
                                                              documentContent);
      documentManager.checkIn(documentId, documentPath, user);

   }

   private void deleteDocument(HttpPresentationComms comms, String documentId, String user)
      throws BaseException {

      documentManager.delete(documentId, user);

   }

   private void massDeleteDocument(HttpPresentationComms comms, String user)
      throws BaseException,
         HttpPresentationException {

      Enumeration en = comms.request.getParameterNames();
      while (en.hasMoreElements()) {
         String name = (String) en.nextElement();
         if (name.startsWith(EnhydraDMConstants.HTML_NAME_SELECT_DOC_CHECKBOX)) {
            String documentId = comms.request.getParameter(name);
            if (documentId != null && !documentId.equals(""))
               documentManager.delete(documentId, user);
         }
      }
   }

   private void createVersion(HttpPresentationComms comms, String documentId, String user)
      throws BaseException {

      documentManager.createVersion(documentId, user);

   }

   private void checkoutDocument(HttpPresentationComms comms,
                                 String documentId,
                                 String user) throws BaseException {
      try {
         Document document = documentManager.checkOut(documentId, user);

         HttpPresentationResponse response = comms.response;
         HttpPresentationOutputStream out = response.getOutputStream();

         byte[] documentContent = documentStore.loadDocumentContent(document.getFilepath());

         response.setContentType(document.getMimeType()
                                 + "; charset=" + EnhydraDMConstants.ENCODING);
         response.setHeader(EnhydraDMConstants.HEAD_CONTENT_DISPOSITION,
                            "attachment; filename=\""
                                  + URLEncoder.encode(document.getName(),
                                                      EnhydraDMConstants.ENCODING) + "\"");
         response.setHeader(EnhydraDMConstants.HEAD_PRAGMA, "public");
         response.setHeader(EnhydraDMConstants.HEAD_CACHE_CONTROL, "max-age=0");
         response.setStatus(HttpPresentationResponse.SC_OK);

         out.write(documentContent);
         out.flush();
      } catch (Exception e) {
         throw new ActionNotAllowedException(e);
      }

   }

   private void unCheckoutDocument(HttpPresentationComms comms,
                                   String documentId,
                                   String user) throws BaseException {

      documentManager.unCheckOut(documentId, user);

   }

   private void switchVersionable(HttpPresentationComms comms,
                                  String documentId,
                                  String user) throws BaseException {

      documentManager.switchAutoVersionable(documentId, user);

   }

   private void switchArchived(HttpPresentationComms comms, String documentId, String user)
      throws BaseException {

      documentManager.switchArchived(documentId, user);

   }

   private void duplicateDocument(HttpPresentationComms comms,
                                  String documentId,
                                  String user) throws BaseException {

      Document doc = documentManager.getDocument(documentId);

      byte[] documentContent = documentStore.loadDocumentContent(doc.getFilepath());
      String documentPath = documentStore.saveDocumentContent(doc.getName(),
                                                              documentContent);
      documentManager.create(doc.getName(),
                             documentPath,
                             doc.getMimeType(),
                             user,
                             (null != doc.getFoDocumentRef() ? doc.getFoDocumentRef()
                                .getId() : null),
                             doc.isTemplate(),
                             (null != doc.getTemplateRef() ? doc.getTemplateRef()
                                .getNumber() : null));

   }

   private void create(String documentName,
                       byte[] documentContent,
                       String mimeType,
                       String user,
                       String foDocumentOid,
                       boolean isTemplate,
                       String templateOid) throws BaseException {

      String documentPath = documentStore.saveDocumentContent(documentName,
                                                              documentContent);
      documentManager.create(documentName,
                             documentPath,
                             mimeType,
                             user,
                             foDocumentOid,
                             isTemplate,
                             templateOid);

   }

   private void downloadVersionFile(HttpPresentationComms comms, String documentId)
      throws BaseException {

      try {
         if (null != comms.application.getLogChannel()) {
            comms.application.getLogChannel().write(Logger.ERROR,
                                                    "Download document version for id: "
                                                          + documentId + "!");
         }
         Document document = documentManager.getDocument(documentId);
         byte[] fileForDownload = documentStore.loadDocumentContent(document.getFilepath());

         HttpPresentationResponse response = comms.response;
         HttpPresentationOutputStream out = response.getOutputStream();
         response.setContentType(new MimetypesFileTypeMap().getContentType(document.getName()));
         response.setHeader(EnhydraDMConstants.HEAD_CONTENT_DISPOSITION,
                            "attachment; filename=\"" + document.getName() + "\"");
         response.setHeader(EnhydraDMConstants.HEAD_PRAGMA, "public");
         response.setHeader(EnhydraDMConstants.HEAD_CACHE_CONTROL, "max-age=0");
         response.setStatus(HttpPresentationResponse.SC_OK);
         out.write(fileForDownload);
         out.flush();
      } catch (BaseException e) {
         throw e;
      } catch (HttpPresentationException e) {
         throw new ActionNotAllowedException(e);
      } catch (IOException e) {
         throw new ActionNotAllowedException(e);
      }

   }
}
