package org.enhydra.dm.presentation;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.ActionNotAllowedException;
import org.enhydra.dm.util.EnhydraDMConstants;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLAnchorElement;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.logging.Logger;

public class RecycleBin extends Base {

   RecycleBinHTML page;

   /*
    * There is the only function needed in order to be a presentation object.
    */
   public void run(HttpPresentationComms comms) throws Exception {

      page = (RecycleBinHTML) comms.xmlcFactory.create(RecycleBinHTML.class);

      comms.request.getHttpServletRequest().setCharacterEncoding(EnhydraDMConstants.ENCODING);

      String documentid = comms.request.getParameter(EnhydraDMConstants.DOCUMENT_ID);
      String action = comms.request.getParameter(EnhydraDMConstants.ACTION);
      String mode = comms.request.getParameter(EnhydraDMConstants.MODE);

      boolean isTemplate = true;
      if (!EnhydraDMConstants.MODE_TEMPLATE.equals(mode)) {
         mode = EnhydraDMConstants.MODE_DOCUMENT;
         isTemplate = false;
      }

      boolean doCommit = true;
      try {
         initTransaction();
         String user = getUser(comms);
         documentManager = ((EnhydraDM) comms.application).getDocumentManager();

         if (null == user) {
            throw new ActionNotAllowedException("User not registred;");

         }
         if (null != action) {
            if (EnhydraDMConstants.ACTION_RESTORE.equals(action)) {
               restoreDocument(comms, documentid, user);
            }
            throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_RECYCLEBIN
                                                  + "?" + EnhydraDMConstants.MODE + "=" + mode);
         }

         page.setTextUser("User: " + user);

         fillFilesetTable(comms, user, isTemplate);

         comms.response.writeDOM(page);
      } catch (BaseException ex) {
         doCommit = false;
         if (null != comms.application.getLogChannel()) {
            comms.application.getLogChannel().write(Logger.ERROR,
                                                    "Action :"
                                                          + action + " failure! Reason: "
                                                          + ex.getMessage());
         }

         throw new Exception(ex);
      } finally {
         if (doCommit) {
            commitTransaction();
         } else {
            rollbackTransaction();
         }
      }
   }

   private void fillFilesetTable(HttpPresentationComms comms,
                                 String user,
                                 boolean isTemplate) throws BaseException {

      String mode = EnhydraDMConstants.MODE_DOCUMENT;
      if (isTemplate) {
         mode = EnhydraDMConstants.MODE_TEMPLATE;
      }

      Node fileRow = page.getElementDocumentRow();
      Document[] documents = null;
      documents = documentManager.getDeleted(isTemplate);
      SimpleDateFormat sdf = new SimpleDateFormat(EnhydraDMConstants.DATE_FORMAT);

      HTMLAnchorElement restoreLink = page.getElementRestoreLink();
      HTMLAnchorElement logoutLink = page.getElementLogoutLink();
      HTMLAnchorElement backLink = page.getElementBackLink();
      HTMLAnchorElement documentDetailLink = page.getElementDocumentDetailLink();

      logoutLink.setHref(EnhydraDMConstants.NAVIG_MANAGER
                         + "?" + EnhydraDMConstants.ACTION + "=" + EnhydraDMConstants.ACTION_LOGOUT);

      if (isTemplate) {
         page.setTextHeader("Deleted Templates");
      } else {
         page.setTextHeader("Deleted Documents");
      }

      backLink.setHref(EnhydraDMConstants.NAVIG_MANAGER + "?" + EnhydraDMConstants.MODE + "=" + mode);

      if (documents != null) {

         for (int i = 0; i < documents.length; i++) {
            String documentName = documents[i].getName();
            String documentId = documents[i].getNumber().toString();
            long documentSize = documents[i].getSize();

            String appUser = documents[i].getDeletedBy();

            documentDetailLink.setHref(EnhydraDMConstants.NAVIG_DETAILS
                                       + "?" + EnhydraDMConstants.DOCUMENT_ID + "=" + documentId
                                       + "&" + EnhydraDMConstants.MODE + "=" + mode);
            page.setTextDocumentName(documentName);
            page.setTextDocumentId(documentId);
            page.setTextDocumentSize(String.valueOf(documentSize));

            page.setTextDeletedBy(appUser);

            Date mainDate = new Date(documents[i].getDeletedDate());
            if (mainDate != null) {
               String strMainDate = sdf.format(mainDate);
               page.setTextDeletionDate(strMainDate);
            } else {
               page.setTextDeletionDate("-");
            }

            restoreLink.setHref(EnhydraDMConstants.NAVIG_RECYCLEBIN
                                + "?" + EnhydraDMConstants.ACTION + "="
                                + EnhydraDMConstants.ACTION_RESTORE + "&"
                                + EnhydraDMConstants.DOCUMENT_ID + "=" + documents[i].getNumber()
                                + "&" + EnhydraDMConstants.MODE + "=" + mode);

            fileRow.getParentNode().insertBefore(fileRow.cloneNode(true), fileRow);
         }
      }
      fileRow.getParentNode().removeChild(fileRow);
   }

   private void restoreDocument(HttpPresentationComms comms,
                                String documentId,
                                String user) throws BaseException {

      documentManager.restore(documentId, user);
   }
}
