package org.enhydra.dm.presentation;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.UserManager;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.ActionNotAllowedException;
import org.enhydra.dm.util.EnhydraDMConstants;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;

/**
 * Register.java handles the user registration functionality of the DiscRack app.
 */
public class Register extends Base {

   RegisterHTML page;
   
   UserManager userManager = null;

   public void run(HttpPresentationComms comms) throws Exception {

      comms.request.getHttpServletRequest().setCharacterEncoding(EnhydraDMConstants.ENCODING);

      page = (RegisterHTML) comms.xmlcFactory.create(RegisterHTML.class);
      boolean doCommit = true;
      try {
         initTransaction();
         
         String action = comms.request.getParameter(EnhydraDMConstants.ACTION);
         
         if (null != action) {
            documentManager = ((EnhydraDM) comms.application).getDocumentManager();
            userManager = ((EnhydraDM) comms.application).getUserManager();
            registerUser(comms);
         }

         comms.response.writeDOM(page);
      } catch (Exception ex) {
         doCommit = false;
         throw new Exception(ex);
      } finally {
         if (doCommit) {
            commitTransaction();
         } else {
            rollbackTransaction();
         }
      }
   }

   private void registerUser(HttpPresentationComms comms) throws BaseException {
      try {
         String username = comms.request.getParameter(EnhydraDMConstants.PARAM_LOGIN);
         String password = comms.request.getParameter(EnhydraDMConstants.PARAM_PASSWORD);
         String firstname = comms.request.getParameter(EnhydraDMConstants.PARAM_FIRSTNAME);
         String lastname = comms.request.getParameter(EnhydraDMConstants.PARAM_LASTNAME);
         String repassword = comms.request.getParameter(EnhydraDMConstants.PARAM_REPASSWORD);

         // if login or password field is empty, generate error and redirect to this PO
         if (username.length() == 0
             || password.length() == 0 || firstname.length() == 0
             || lastname.length() == 0) {
            page.setTextErrorText("Missing information. Please make sure all fields are filled out exactly");

            return;
         }

         // Check that password was duplicated correctly
         if (!repassword.equals(password)) {
            page.setTextErrorText("Please make sure your password and password confirmation match exactly");
            return;
         }
         if (userManager!=null && userManager.createUser(firstname, lastname, username, password)) {
            throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_LOGIN);
         } else {
            page.setTextErrorText("User with this username already exists! Please try again!");

         }
      } catch (Exception e) {
         throw new ActionNotAllowedException(e);
      }
   }
}
