package org.enhydra.dm.presentation;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.activation.MimetypesFileTypeMap;

import org.enhydra.dm.EnhydraDM;
import org.enhydra.dm.api.DocumentVersion;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.ActionNotAllowedException;
import org.enhydra.dm.util.EnhydraDMConstants;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLAnchorElement;
import org.w3c.dom.html.HTMLImageElement;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;
import com.lutris.appserver.server.httpPresentation.HttpPresentationResponse;
import com.lutris.logging.Logger;

public class Versions extends Base {

   VersionsHTML page;

   /*
    * There is the only function needed in order to be a presentation object.
    */
   public void run(HttpPresentationComms comms) throws Exception {

      page = (VersionsHTML) comms.xmlcFactory.create(VersionsHTML.class);

      comms.request.getHttpServletRequest().setCharacterEncoding(EnhydraDMConstants.ENCODING);

      String documentId = comms.request.getParameter(EnhydraDMConstants.DOCUMENT_ID);
      String versionId = comms.request.getParameter(EnhydraDMConstants.VERSION_ID);
      String action = comms.request.getParameter(EnhydraDMConstants.ACTION);
      String mode = comms.request.getParameter(EnhydraDMConstants.MODE);

      if (!EnhydraDMConstants.MODE_TEMPLATE.equals(mode)) {
         mode = EnhydraDMConstants.MODE_DOCUMENT;
      }

      String user = getUser(comms);

      boolean doCommit = true;
      try {
         initTransaction();
         documentManager = ((EnhydraDM) comms.application).getDocumentManager();
         documentStore = ((EnhydraDM) comms.application).getDocumentStore();

         if (null == user) {
            if (null != comms.application.getLogChannel()) {
               comms.application.getLogChannel().write(Logger.ERROR,
                                                       "User Id parameter in empty!");
            }
            throw new ActionNotAllowedException("User oid parameter is empty;");
         }
         if (action != null && !"".equals(action)) {
            if (EnhydraDMConstants.ACTION_DOWNLOAD.equals(action)) {
               downloadVersionFile(comms, versionId);
               throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_VERSIONS
                                                     + "?" + EnhydraDMConstants.DOCUMENT_ID + "="
                                                     + documentId + "&" + EnhydraDMConstants.MODE
                                                     + "=" + mode);
            } else if (EnhydraDMConstants.ACTION_RESTORE.equals(action)) {
               restoreDocument(comms, versionId, user);
               throw new ClientPageRedirectException(EnhydraDMConstants.NAVIG_MANAGER
                                                     + "?" + EnhydraDMConstants.MODE + "=" + mode);
            }
         }

         page.setTextUser("User: " + user);
         fillFilesetTable(comms, documentId, user, mode);
         comms.response.writeDOM(page);
      } catch (BaseException ex) {
         doCommit = false;
         if (null != comms.application.getLogChannel()) {
            comms.application.getLogChannel().write(Logger.ERROR,
                                                    "Action :"
                                                          + action + " failure! Reason: "
                                                          + ex.getMessage());
         }
         throw new Exception(ex);
      } finally {
         if (doCommit) {
            commitTransaction();
         } else {
            rollbackTransaction();
         }
      }
   }

   private void fillFilesetTable(HttpPresentationComms comms,
                                 String id,
                                 String user,
                                 String mode) throws BaseException {

      Node fileRow = page.getElementDocumentRow();
      DocumentVersion[] documentVers = null;
      documentVers = documentManager.getDocumentVersions(id);

      HTMLAnchorElement readLink = page.getElementReadLink();
      HTMLAnchorElement downloadLink = page.getElementDownloadLink();
      HTMLAnchorElement logoutLink = page.getElementLogoutLink();
      HTMLAnchorElement restoreLink = page.getElementRestoreLink();
      HTMLAnchorElement c2cLink = page.getElementC2cLink();
      HTMLAnchorElement saaLink = page.getElementSaaLink();
      HTMLAnchorElement sabLink = page.getElementSabLink();
      HTMLAnchorElement backLink = page.getElementBackLink();

      HTMLImageElement restoreImage = page.getElementRestoreImage();

      logoutLink.setHref(EnhydraDMConstants.NAVIG_MANAGER
                         + "?" + EnhydraDMConstants.ACTION + "=" + EnhydraDMConstants.ACTION_LOGOUT);
      backLink.setHref(EnhydraDMConstants.NAVIG_MANAGER + "?" + EnhydraDMConstants.MODE + "=" + mode);

      SimpleDateFormat sdf = new SimpleDateFormat(EnhydraDMConstants.DATE_FORMAT);
      if (documentVers != null) {
         for (int i = 0; i < documentVers.length; i++) {
            page.setTextDocumentName(documentVers[i].getDocument().getName());
            page.setTextDocumentId(id);
            page.setTextVersionId(documentVers[i].getNumber());
            page.setTextDocumentSize(String.valueOf(documentVers[i].getSize()));
            page.setTextVersionNumber(documentVers[i].getVersionNumber());

            Date mainDate = new Date(documentVers[i].getLastModifiedDate());
            if (mainDate != null) {
               String strMainDate = sdf.format(mainDate);
               page.setTextDocumentLastModifiedDate(strMainDate);
            } else {
               page.setTextDocumentLastModifiedDate("-");
            }

            Date versDate = new Date(documentVers[i].getVersionDate());
            if (versDate != null) {
               String strVersDate = sdf.format(versDate);
               page.setTextVersionDate(strVersDate);
            } else {
               page.setTextVersionDate("-");
            }

            String parameters = null;

            parameters = EnhydraDMConstants.ACTION
                         + "=" + EnhydraDMConstants.ACTION_READ_VERSION + "&"
                         + EnhydraDMConstants.VERSION_ID + "=" + documentVers[i].getNumber();
            if (EnhydraDM.desenc != null) {
               parameters = EnhydraDM.desenc.encrypt(parameters);
               parameters = EnhydraDMConstants.PARAM + "=" + parameters;
            }

            readLink.setHref(EnhydraDMConstants.NAVIG_WEBDAV
                             + "/" + documentVers[i].getDocument().getName() + "?"
                             + parameters);
            downloadLink.setHref(EnhydraDMConstants.NAVIG_VERSIONS
                                 + "?" + EnhydraDMConstants.ACTION + "="+EnhydraDMConstants.ACTION_DOWNLOAD+"&"
                                 + EnhydraDMConstants.VERSION_ID + "=" + documentVers[i].getNumber()
                                 + "&" + EnhydraDMConstants.DOCUMENT_ID + "=" + id + "&" + EnhydraDMConstants.MODE + "=" + mode);

            c2cLink.setHref("javascript: copy2clipboard('"
                            + EnhydraDMConstants.NAVIG_WEBDAV + "/"
                            + documentVers[i].getDocument().getName() + "?" + parameters
                            + "');");
            saaLink.setHref("javascript: sendasattachement('"
                            + EnhydraDMConstants.NAVIG_WEBDAV + "/"
                            + documentVers[i].getDocument().getName() + "?" + parameters
                            + "');");
            sabLink.setHref("javascript: sendShortcut('"
		                    + EnhydraDMConstants.NAVIG_WEBDAV + "/"
		                    + documentVers[i].getDocument().getName() + "?" + parameters
		                    + "');");            

            if (documentVers[i].getDocument().getLockedBy() != null
                || documentVers[i].getDocument().getCheckedOutBy() != null) {

               restoreLink.removeAttribute("href");
               restoreImage.setSrc("media/restoreDis.gif");

            } else {
               String value = EnhydraDMConstants.NAVIG_VERSIONS
                              + "?" + EnhydraDMConstants.ACTION + "="+EnhydraDMConstants.ACTION_RESTORE+"&"
                              + EnhydraDMConstants.VERSION_ID + "=" + documentVers[i].getNumber()
                              + "&" + EnhydraDMConstants.DOCUMENT_ID + "=" + id + "&" + EnhydraDMConstants.MODE + "=" + mode;
               restoreLink.setAttribute("href", value);

               restoreImage.setSrc("media/restore.gif");

            }

            fileRow.getParentNode().insertBefore(fileRow.cloneNode(true), fileRow);
         }
      }
      fileRow.getParentNode().removeChild(fileRow);
   }

   private void restoreDocument(HttpPresentationComms comms, String versionId, String user)
      throws BaseException {
      documentManager.restoreDocumentVersion(versionId, user);
   }

   private void downloadVersionFile(HttpPresentationComms comms, String versionId)
      throws BaseException {

      try {
         if (null != comms.application.getLogChannel()) {
            comms.application.getLogChannel().write(Logger.ERROR,
                                                    "Download document version for id: "
                                                          + versionId + "!");
         }
         DocumentVersion documentVersion = documentManager.getDocumentVersion(versionId);
         byte[] fileForDownload = documentStore.loadDocumentContent(documentVersion.getFilepath());

         HttpPresentationResponse response = comms.response;
         HttpPresentationOutputStream out = response.getOutputStream();
         response.setContentType(new MimetypesFileTypeMap().getContentType(documentVersion.getDocument()
            .getName()));
         response.setHeader(EnhydraDMConstants.HEAD_CONTENT_DISPOSITION, "attachment; filename=\""
                                                   + documentVersion.getDocument()
                                                      .getName() + "\"");
         response.setHeader(EnhydraDMConstants.HEAD_PRAGMA, "public");
         response.setHeader(EnhydraDMConstants.HEAD_CACHE_CONTROL, "max-age=0");
         response.setStatus(HttpPresentationResponse.SC_OK);
         out.write(fileForDownload);
         out.flush();
      } catch (BaseException e) {
         throw e;
      } catch (HttpPresentationException e) {
         throw new ActionNotAllowedException(e);
      } catch (IOException e) {
         throw new ActionNotAllowedException(e);
      }

   }
}
