/**
 * 
 */
package org.enhydra.dm;

import java.io.IOException;
import java.lang.reflect.Array;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.UserTransaction;

import org.enhydra.dm.api.DocumentManager;
import org.enhydra.dm.api.DocumentStore;
import org.enhydra.dm.api.handler.MethodHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.api.util.DesEncription;
import org.enhydra.dm.handler.DefaultCheckinHandler;
import org.enhydra.dm.handler.DefaultCheckoutHandler;
import org.enhydra.dm.handler.DefaultCopyHandler;
import org.enhydra.dm.handler.DefaultDeleteHandler;
import org.enhydra.dm.handler.DefaultGetHandler;
import org.enhydra.dm.handler.DefaultHeadHandler;
import org.enhydra.dm.handler.DefaultLockHandler;
import org.enhydra.dm.handler.DefaultMkcolHandler;
import org.enhydra.dm.handler.DefaultMoveHandler;
import org.enhydra.dm.handler.DefaultOptionsHandler;
import org.enhydra.dm.handler.DefaultPostHandler;
import org.enhydra.dm.handler.DefaultPropfindHandler;
import org.enhydra.dm.handler.DefaultProppatchHandler;
import org.enhydra.dm.handler.DefaultPutHandler;
import org.enhydra.dm.handler.DefaultReportHandler;
import org.enhydra.dm.handler.DefaultUncheckoutHandler;
import org.enhydra.dm.handler.DefaultUnlockHandler;
import org.enhydra.dm.handler.DefaultVersionHandler;

import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.httpPresentation.servlet.HttpPresentationServlet;
import com.lutris.util.ConfigException;

/**
 * @author Slobodan Vujasinovic
 */
public class WebDavServlet extends HttpServlet {

   private final Map handlers = new HashMap();

   /**
    * Transaction object reference
    */
   protected UserTransaction transaction = null;

   public static String transactionJNDIPath;

   public static final String defaultLogImpl = "org.enhydra.dm.api.loggers.DefaultLog";

   public static final String defaultDocumentManagerImpl = "org.enhydra.dm.business.DocumentManagerImpl";

   public static final String defaultDocumentStoreImpl = "org.enhydra.dm.business.DocumentStoreImpl";

   public static final String defaultUserManagerImpl = "org.enhydra.dm.business.UserManagerImpl";

   public static final String defaultDesEncriptionImpl = null;

   private Log logger = null;

   /**
    * DesEncription
    */
   private DesEncription desenc = null;

   /**
    * Document Manager
    */
   private DocumentManager documentManager = null;

   /**
    * Document Store
    */
   private DocumentStore documentStore = null;

   /**
    * Enhydra Super Servlet
    */
   private HttpPresentationServlet hps;

   /**
    * Method for initialization ; Reads parameters from config and initialize application
    * parts
    */
   public void init() throws ServletException {
      ServletConfig config = getServletConfig();
      Properties props = null;
      try {
         props = readConfigIntoProp(config);
         transactionJNDIPath = config.getInitParameter("transactionJNDIPath");

      } catch (Exception ex) {
         throw new ServletException(ex.getMessage());
      }
      initLogger(props);
      if (null != logger) {
         logger.log(Log.INFORMATION,
                    "Logger for WebDavServlet is successfully initialized!");
      }
      initDocumentManager(props);
      initDocumentStore(props);
      initHandlers(config);
      initDesEncriptor(props);
      
      /*
       * Enhydra Super Servlet Initialization
       */
      hps = new HttpPresentationServlet();
      hps.init(config);
   }

   /**
    * Logger initialization. Try to find logProvider parameter(full class name for Log
    * class implementation) in config. If parameter does not exist application uses
    * default implementation: DefaultLog.
    * 
    * @param config
    * @throws ServletException
    */
   private void initLogger(Properties props) throws ServletException {
      try {
         String logProviderName = Log.class.getName();
         String logProvider = props.getProperty(logProviderName, defaultLogImpl);

         try {

            logger = (Log) Class.forName(logProvider).newInstance();
            Log.setLogger(logger);
         } catch (Exception ex) {

            logger = Log.getInstance();
         }
      } catch (Exception e) {
         throw new ServletException(e);
      }

   }

   /**
    * Manager initialization. Try to find full class name for Manager interface
    * implementation in config. If parameter does not exist application uses default
    * implementation: DocumentManagerImpl
    * 
    * @param config
    * @throws ServletException
    */
   private void initDocumentManager(Properties props) throws ServletException {

      try {

         String documentManagerImplClassName = props.getProperty(DocumentManager.class.getName(),
                                                                 defaultDocumentManagerImpl);

         documentManager = (DocumentManager) Class.forName(documentManagerImplClassName)
            .newInstance();

         documentManager.setLogger(logger);
         documentManager.configure(props);

      } catch (Exception e) {
         throw new ServletException(e);
      }
   }

   /**
    * DocumentStore initialization. Try to find full class name for DocumentStore
    * interface implementation in config. If parameter does not exist application uses
    * default implementation: DocumentStoreImpl
    * 
    * @param config
    * @throws ServletException
    */
   private void initDocumentStore(Properties props) throws ServletException {

      try {

         String documentStoreImplClassName = props.getProperty(DocumentStore.class.getName(),
                                                               defaultDocumentStoreImpl);

         documentStore = (DocumentStore) Class.forName(documentStoreImplClassName)
            .newInstance();

         documentStore.setLogger(logger);
         documentStore.configure(props);

      } catch (Exception e) {
         throw new ServletException(e);
      }
   }

   /**
    * DesEncriptor initialization. Try to find full class name for DesEncriptor interface
    * implementation in config. If parameter does not exist application uses default
    * implementation: DesEncriptorImpl
    * 
    * @param config
    * @throws ServletException
    */
   private void initDesEncriptor(Properties props) throws ServletException {
      try {

         String desEncriptionImplClass = props.getProperty(DesEncription.class.getName(),
                                                           defaultDesEncriptionImpl);
         if (null != desEncriptionImplClass) {
            desenc = (DesEncription) Class.forName(desEncriptionImplClass).newInstance();
            if (null != desenc) {
               desenc.setLogger(logger);
               desenc.configure(props);
            } else {
               if (null != logger) {
                  logger.log(Log.WARNING, "Can't find DesEncriptor implementation class!");
               }
            }
         }

      } catch (Exception ex) {
         throw new ServletException(ex);
      }
   }

   /**
    * Destroy method for all handler implementations
    */
   public void destroy() {
      Iterator iterator = handlers.entrySet().iterator();
      while (iterator.hasNext()) {
         ((MethodHandler) ((Map.Entry) iterator.next()).getValue()).destroy();
         iterator.remove();
      }
      hps.destroy();
   }

   /**
    * Service method for WebDavServlet
    * 
    * @param request The request being handled.
    * @param response The response supplied by the servlet.
    * @throws IOException If an IO error occurs while handling the request.
    * @throws ServletException If an application error occurs.
    */
   protected void service(HttpServletRequest request, HttpServletResponse response)
      throws IOException,
         ServletException {

      // Registering Thread to enhydra (getting correct DatabaseManager instance) - DODS usage obligation
      Enhydra.register(hps.getApplication());
      try {
         serviceRequest(request, response);
      } finally {
         // Be sure the thread is unregistered no matter what.
         Enhydra.unRegister();
      }
   }

   /**
    * ServiceRequest method for WebDavServlet (Calls proper Handler for every request)
    * 
    * @param request The request being handled.
    * @param response The response supplied by the servlet.
    * @throws IOException If an IO error occurs while handling the request.
    * @throws ServletException If an application error occurs.
    */
   protected void serviceRequest(HttpServletRequest request, HttpServletResponse response)
      throws IOException,
         ServletException {
      boolean doCommit = true;
      try {
         initTransaction();
         MethodHandler handler = getHandler(request.getMethod());
         if (handler != null) {
            try {
               handler.service(request, response);
            } catch (Throwable throwable) {
               if (null != logger) {
                  logger.log(Log.ERROR, request.getMethod()
                                        + " request failure! Reason: "
                                        + throwable.getMessage());
               }
               fail(request, response);
            }
         } else {
            if (null != logger) {
               logger.log(Log.ERROR, "Method " + request.getMethod() + " not allowed");
            }
            response.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED);
         }
      } catch (Exception ex) {
         doCommit = false;
         throw new ServletException(ex);
      } finally {
         if (doCommit) {
            try {
               commitTransaction();
            } catch (Exception e) {
               e.printStackTrace();
            }
         } else {
            try {
               rollbackTransaction();
            } catch (Exception e) {
               e.printStackTrace();
            }
         }
      }
   }

   /**
    * Returns the <code>MethodHandler</code> for the specified method.
    * 
    * @param method The HTTP method (GET, POST, PUT, etc.) being handled.
    * @return A <code>MethodHandler</code> capable of servicing the request using the
    *         given method.
    */
   protected MethodHandler getHandler(String method) {
      if (method == null)
         return null;
      return (MethodHandler) handlers.get(method.toUpperCase());
   }

   /**
    * Method for handler initialization. First initialize default implementations then
    * read config and initialize other implementations; Fill handlers map.
    * 
    * @param config
    * @throws ServletException
    */
   private void initHandlers(ServletConfig config) throws ServletException {
      handlers.clear();
      // WebDav
      handlers.put("OPTIONS", new DefaultOptionsHandler());
      handlers.put("HEAD", new DefaultHeadHandler());
      handlers.put("GET", new DefaultGetHandler());
      handlers.put("POST", new DefaultPostHandler());
      handlers.put("DELETE", new DefaultDeleteHandler());
      handlers.put("PROPFIND", new DefaultPropfindHandler());
      handlers.put("PROPPATCH", new DefaultProppatchHandler());
      handlers.put("COPY", new DefaultCopyHandler());
      handlers.put("MOVE", new DefaultMoveHandler());
      handlers.put("PUT", new DefaultPutHandler());
      handlers.put("MKCOL", new DefaultMkcolHandler());

      handlers.put("LOCK", new DefaultLockHandler());
      handlers.put("UNLOCK", new DefaultUnlockHandler());

      // DeltaV
      handlers.put("VERSION-CONTROL", new DefaultVersionHandler());
      handlers.put("CHECKOUT", new DefaultCheckoutHandler());
      handlers.put("UNCHECKOUT", new DefaultUncheckoutHandler());
      handlers.put("CHECKIN", new DefaultCheckinHandler());
      handlers.put("REPORT", new DefaultReportHandler());

      Enumeration parameters = config.getInitParameterNames();
      while (parameters.hasMoreElements()) {
         String name = (String) parameters.nextElement();
         if (!name.startsWith("handler."))
            continue;
         String method = name.substring(8);
         try {
            handlers.put(method.toUpperCase(),
                         Class.forName(config.getInitParameter(name)).newInstance());
         } catch (Exception e) {
            throw new ServletException(e);
         }
      }
      Iterator iterator = handlers.values().iterator();
      while (iterator.hasNext()) {
         ((MethodHandler) iterator.next()).init(config,
                                                documentManager,
                                                documentStore,
                                                desenc,
                                                logger);
      }
   }

   /**
    * Method that handle exceptions from all handler implementations
    * 
    * @param request
    * @param response
    * @throws ServletException
    * @throws IOException
    */
   private void fail(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {

      response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
      response.flushBuffer();
   }

   /**
    * Method initializes DB transation instance
    */
   protected void initTransaction() throws Exception {
      Context ctx = new InitialContext();
      transaction = (UserTransaction) ctx.lookup(transactionJNDIPath);
      if (transaction != null) {
         transaction.begin();
      }
   }

   /**
    * Method commits DB transation instance
    */
   protected void commitTransaction() throws Exception {
      if (transaction != null) {
         transaction.commit();
      }

   }

   private void rollbackTransaction() throws Exception {
      if (transaction != null) {
         transaction.rollback();
      }
   }

   /**
    * Methos that generate correspondent Properties object for given ServletConfig object
    * 
    * @param config
    * @return
    * @throws ConfigException
    */
   private Properties readConfigIntoProp(ServletConfig config) throws ConfigException {
      Properties props = new Properties();
      try {
         Enumeration leafKeys = config.getInitParameterNames();

         while (leafKeys.hasMoreElements()) {
            String leafKey = (String) leafKeys.nextElement();
            Object value = config.getInitParameter(leafKey);
            if (value.getClass().isArray()) {
               int len = Array.getLength(value);
               String valueArray = Array.get(value, 0).toString();
               for (int j = 1; j < len; j++) {
                  valueArray = valueArray + "," + Array.get(value, j).toString();
               }
               props.put(leafKey + "[]", valueArray);
            } else {
               if (value instanceof javax.sql.DataSource) {
                  props.put(leafKey, value.toString());
               } else {
                  props.put(leafKey, value);
               }
            }

         }

      } catch (Exception ex) {
         throw new ConfigException(ex);
      }
      return props;
   }
}
