/**
 * 
 */
package org.enhydra.dm.api;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.data.DocumentDO;

/**
 * Document interface that represents association with DocumentDO object with additional
 * business functionality extensions.
 * 
 * @author Slobodan Vujasinovic
 */
public interface Document {

   /**
    * Returns the entity tag for the specified resource. The returned string uniquely
    * identifies the current incarnation of the given resource.
    * 
    * @param file The resource whose entity tag is to be retrieved.
    * @return A <code>String</code> containing the entity tag for the resource.
    */
   public String getETag();

   /**
    * Formats a timestamp (representing milliseconds since the epoch) as used in the
    * WebDAV <code>getlastmodified</code> property.
    * 
    * @return A <code>String</code> containing the formatted result.
    */
   public String getLastModifiedFormated();

   public DocumentDO createDO() throws BaseException;

   public DocumentVersion createVersion();

   public boolean isArchived();

   public void setArchived(boolean archived);

   public boolean isAutoversionable();

   public void setAutoversionable(boolean autoversionable);

   public String getCheckedOutBy();

   public void setCheckedOutBy(String checkedOutBy);

   public String getCreatedBy();

   public void setCreatedBy(String createdBy);

   public long getCreatedDate();

   public void setCreatedDate(long createdDate);

   public String getCurrentVersionNumber();

   public void setCurrentVersionNumber(String currentVersionNumber);

   public String getFilepath();

   public void setFilepath(String filepath);

   public FoDocument getFoDocumentRef();

   public void setFoDocumentRef(FoDocument foDocumentRef);

   public String getLastModifiedBy();

   public void setLastModifiedBy(String lastModifiedBy);

   public long getLastModifiedDate();

   public void setLastModifiedDate(long lastModifiedDate);

   public String getLockedBy();

   public void setLockedBy(String lockedBy);

   public String getMimeType();

   public void setMimeType(String mimeType);

   public String getName();

   public void setName(String name);

   public String getNumber();

   public String getId();

   public void setNumber(String number);

   public void setId(String id);

   public long getSize();

   public void setSize(long size);

   public boolean isTemplate();

   public void setTemplate(boolean template);

   public Document getTemplateRef();

   public void setTemplateRef(Document templateRef);

   public String getDeletedBy();

   public void setDeletedBy(String deletedBy);

   public long getDeletedDate();

   public void setDeletedDate(long deletedDate);
}
