package org.enhydra.dm.api;

import java.util.Properties;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;

public interface DocumentStore {

   /**
    * Saving document content into document store for the first time
    * 
    * @param documentName
    * @param documentContent
    * @return
    * @throws BaseException
    */
   public String saveDocumentContent(String documentName, byte[] documentContent)
      throws BaseException;

   /**
    * Saving document content into document store - overwrite existing
    * 
    * @param documentName
    * @param documentContent
    * @param path
    * @return
    * @throws BaseException
    */
   public String saveDocumentContent(String documentName,
                                     byte[] documentContent,
                                     String path) throws BaseException;

   /**
    * Load document content from document store.
    * 
    * @param documentFilePath
    * @return
    * @throws BaseException
    */
   public byte[] loadDocumentContent(String documentFilePath) throws BaseException;

   /**
    * Getter for logger.
    * 
    * @return
    */
   public Log getLogger();

   /**
    * Setter for logger.
    * 
    * @param logger
    */
   public void setLogger(Log logger);

   /**
    * Getter for document store path
    * 
    * @return
    */
   public String getDocStorePath();

   /**
    * Setter for document store path
    * 
    * @param docStorePath
    */
   public void setDocStorePath(String docStorePath);

   /**
    * Configure method that reads parameters from Properties
    * 
    * @param properties
    * @throws BaseException
    */
   public void configure(Properties properties) throws BaseException;

}
