// A simple example to illustrate properties reloading.
package org.enhydra.dm.api.util.filesys;

import java.io.FileNotFoundException;

import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.filesys.FileMonitor;

public abstract class FileChangeListenerImpl implements FileChangeListener {
   protected Log logger;

   public FileChangeListenerImpl(String fileName, long fileRefreshPeriod, Log logger) throws FileNotFoundException {
      this.logger = logger; 
      fileChanged(fileName);
      setFileRefreshPeriod(fileRefreshPeriod, fileName);
   }

   /**
    * Method connects file and its change listener and set <code>fileRefreshPeriod</code>
    * if <code>fileRefreshPeriod > 0</code>. If <code>fileRefreshPeriod < 0</code>
    * Loaded file won't be reloaded at runtime.  
    * 
    * @param fileRefreshPeriod
    * @param fileName
    * @throws FileNotFoundException
    */
   public synchronized void setFileRefreshPeriod(long fileRefreshPeriod, String fileName)
      throws FileNotFoundException {
      if (fileRefreshPeriod > 0) {
         FileMonitor.getInstance().addFileChangeListener(this,
                                                         fileName,
                                                         fileRefreshPeriod);
         if (logger != null) {
            logger.log(Log.INFORMATION, "File refresh period is setup:"
                                        + fileRefreshPeriod + "ms.");
         }
      } else {
         if (logger != null) {
            logger.log(Log.INFORMATION, "Bad file refresh period:"
                                        + fileRefreshPeriod
                                        + ". Loaded file won't be reloaded at runtime.");
         }
      }
   }

   public abstract void fileChanged(String fileName);

   public Log getLogger() {
      return logger;
   }

   public void setLogger(Log logger) {
      this.logger = logger;
   }
}
