/**
 * 
 */

package org.enhydra.dm.handler;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.handler.AbstractHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * Default implementation of a handler for requests using the WebDAV LOCK method.
 * 
 * @author Slobodan Vujasinovic
 */
public class DefaultLockHandler extends AbstractHandler {

   /**
    * Services requests which use the WebDAV LOCK method.
    * 
    * @param request The request being serviced.
    * @param response The servlet response.
    * @throws ServletException If an application error occurs.
    * @throws IOException If an IO error occurs while handling the request.
    */

   public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {
      
      Map map = this.readParameters(request);

      /*
       * User is required for this operation!
       */
      String user = getUser(request);
      if (null == user) {
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "(LOCK)User not defined.");
         }
         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;
      }

      String mod = (String) map.get(EnhydraDMConstants.ACTION);
      if (null == mod) {
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG,
                            "(LOCK)Action parameter isn't defined parameter does not exist.");
         }

         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;
      }

      String id = (String) map.get(EnhydraDMConstants.DOCUMENT_ID);

      try {
         if (mod.equals(EnhydraDMConstants.ACTION_READ)
             || mod.equals(EnhydraDMConstants.ACTION_READ_VERSION)) {

            response.setStatus(HttpServletResponse.SC_OK);
            response.flushBuffer();
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            response.flushBuffer();
            if (null != getLogger()) {
               getLogger().log(Log.INFORMATION, "(LOCK)File is opened in read mode!");
            }

         } else {

            if (null == id) {
               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG,
                                  "(LOCK)Document id parameter does not exist.");
               }
               response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
               response.flushBuffer();
               return;
            }

            if (null != getLogger()) {
               getLogger().log(Log.DEBUG, "LOCK Request for document (id)\"{0}\".", id);
            }

            getDocumentManager().lock(id, user);

            setLockResponse(user, response);

            HttpSession session = request.getSession(true);
            session.setAttribute(SESSION_ATT_LOCKED, "true");

         }
      } catch (BaseException e) {

         response.flushBuffer();
         throw new ServletException(e.getMessage());
      }
      response.flushBuffer();
   }

   private void setLockResponse(String user, HttpServletResponse response)
      throws ServletException,
         IOException,
         BaseException {

      String content = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>"
                       + "<prop xmlns=\"DAV:\">";

      content = content
                + "<lockdiscovery>" + "<activelock>" + "<locktype>" + " <write/>"
                + "</locktype>" + "<lockscope>" + "<exclusive/>" + "<shared/>"
                + "</lockscope>" + "<depth>0</depth>" + "<owner>" + user + "</owner>"
                + "<locktoken/>" + "<timeout>Second-" + getLockRequestTimeout()
                + "</timeout>" + "</activelock>" + "</lockdiscovery>" + "</prop>";

      response.getWriter().write(content);
      response.setStatus(HttpServletResponse.SC_OK);

      response.setContentType("text/xml; charset=\"utf-8\"");
   }

}
