/**
 * 
 */

package org.enhydra.dm.handler;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.handler.AbstractHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * Default implementation of a handler for requests using the WebDAV UNLOCK method.
 * 
 * @author Slobodan Vujasinovic
 */
public class DefaultUnlockHandler extends AbstractHandler {

   /**
    * Services requests which use the WebDAV UNLOCK method.
    * 
    * @param request The request being serviced.
    * @param response The servlet response.
    * @throws ServletException If an application error occurs.
    * @throws IOException If an IO error occurs while handling the request.
    */

   public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {

      response.setHeader(EnhydraDMConstants.HEAD_ALLOW, getAllowedMethods());

      Map map = this.readParameters(request);

      /*
       * User is required for this operation!
       */
      String user = getUser(request);
      if (null == user) {
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "(UNLOCK) User not defined.");
         }
         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;

      }

      String id = (String) map.get(EnhydraDMConstants.DOCUMENT_ID);
      if (null == id) {

         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "(UNLOCK) Document id parameter does not exist.");
         }

         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;
      }

      if (null == request.getSession(true).getAttribute(SESSION_ATT_LOCKED)) {
         if (null != getLogger()) {
            getLogger().log(Log.WARNING, "(UNLOCK) Session Expired.");
         }
         response.setStatus(HttpServletResponse.SC_METHOD_NOT_ALLOWED);
         response.flushBuffer();
         return;
      }

      if (null != getLogger()) {
         getLogger().log(Log.DEBUG, "(UNLOCK) Request for document (id)\"{0}\".", id);
      }

      String path = (String) request.getSession().getAttribute(id);
      try {

         if (path == null || "".equals(path)) {
            getDocumentManager().unlock(id, user);
            response.setStatus(HttpServletResponse.SC_NO_CONTENT);
         } else {
            getDocumentManager().update(id, path, user, null);
            request.getSession().setAttribute(id, null);
            response.setStatus(HttpServletResponse.SC_NO_CONTENT);
         }
      } catch (BaseException e) {

         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         throw new ServletException(e.getMessage());
      }

   }

}
