package org.enhydra.dm.util;

import java.security.spec.AlgorithmParameterSpec;
import java.util.Properties;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.api.util.DesEncription;

import com.lutris.util.Config;

/**
 * Encrypting with DES Using a Pass Phrase This class uses pass phrase (a string of
 * multiple words) for encryption. Web page:
 * http://www.exampledepot.com/egs/javax.crypto/PassKey.html?l=rel
 */

public class DesEncriptionImpl implements DesEncription {

   public Cipher ecipher;

   public Cipher dcipher;

   public final String encodeSecretDefault = "SomeStringValue";

   public final String encodeEncodingDefault = "UTF-8";

   public String encoding;

   public String secret;

   public Log logger;

   // 8-byte Salt
   byte[] salt = {
         (byte) 0xA9,
         (byte) 0x9B,
         (byte) 0xC8,
         (byte) 0x32,
         (byte) 0x56,
         (byte) 0x35,
         (byte) 0xE3,
         (byte) 0x03
   };

   // Iteration count
   int iterationCount = 19;

   /**
    * @param strPass Secret phrase parameter
    * @param strPass Encoding parameter
    * @throws BaseException
    */
   public DesEncriptionImpl(String strPass, String strEncoding) {

      if (strPass != null && !"".equalsIgnoreCase(strPass)) {
         secret = strPass;
      }

      if (strEncoding != null && !"".equalsIgnoreCase(strEncoding)) {
         encoding = strEncoding;
      }

   }

   public DesEncriptionImpl() {

   }

   private void init() throws BaseException {
      try {
         // Create the key
         PBEKeySpec keySpec = new PBEKeySpec(secret.toCharArray(), salt, iterationCount);
         SecretKey key = SecretKeyFactory.getInstance("PBEWithMD5AndDES")
            .generateSecret(keySpec);
         ecipher = Cipher.getInstance(key.getAlgorithm());
         dcipher = Cipher.getInstance(key.getAlgorithm());

         // Prepare the parameter to the ciphers
         AlgorithmParameterSpec paramSpec = new PBEParameterSpec(salt, iterationCount);

         // Create the ciphers
         ecipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
         dcipher.init(Cipher.DECRYPT_MODE, key, paramSpec);

      } catch (Exception e) {
         throw new BaseException(e.getMessage());
      }

   }

   /**
    * @param strPass Secret phrase parameter
    * @throws BaseException
    */
   public DesEncriptionImpl(String strPass) {
      this(strPass, null);
   }

   /**
    * Encrypt the parametar
    * 
    * @param str parametar to encrypt
    * @param strEncoding type of encoding
    * @return encrypted object
    * @throws BaseException
    */
   public String encrypt(String str, String strEncoding) throws BaseException {

      if (null != logger) {
         logger.log(Log.DEBUG, "Encrypt method for parameters!");
      }
      try {

         byte[] input = str.getBytes(strEncoding);

         // Encrypt
         byte[] encript = ecipher.doFinal(input);

         // Encode bytes to base64 to get a string
         return Base64.encodeBytes(encript, 16);

      } catch (Exception e) {
         throw new BaseException(e.getMessage());
      }

   }

   /**
    * Encrypt the parametar
    * 
    * @param str parametar to encrypt
    * @return encrypted object
    * @throws BaseException
    */
   public String encrypt(String str) throws BaseException {
      return encrypt(str, encoding);
   }

   /**
    * Decrypt the parametar
    * 
    * @param strEncripted encrypted parametar
    * @param strEncoding type of encoding
    * @return decrypted object
    * @throws BaseException
    */
   public String decrypt(String strEncrypted, String strEncoding) throws BaseException {

      if (null != logger) {
         logger.log(Log.DEBUG, "Decrypt method for parameters!");
      }
      try {
         // Decode base64 to get bytes
         byte[] input = Base64.decode(strEncrypted, 16);

         // Decrypt
         byte[] decrypted = dcipher.doFinal(input);

         // Decode
         return new String(decrypted, strEncoding);

      } catch (Exception e) {
         throw new BaseException(e.getMessage());
      }

   }

   /**
    * Decrypt the parametar
    * 
    * @param strEncripted encrypted parametar
    * @return decrypted object
    * @throws BaseException
    */
   public String decrypt(String strEncrypted) throws BaseException {
      return decrypt(strEncrypted, encoding);
   }

   public static void main(String[] args) {
      DesEncriptionImpl de;
      try {
         de = new DesEncriptionImpl("SomeStringValue");
         de.init();
         String parameter = EnhydraDMConstants.ACTION
                            + "=edit&" + EnhydraDMConstants.DOCUMENT_ID + "=1000000";
         System.out.println("1. " + parameter);
         parameter = de.encrypt(parameter);
         System.out.println("2. " + parameter);
         parameter = de.decrypt(parameter);
         System.out.println("3. " + parameter);
         System.out.println();
      } catch (BaseException e) {

         e.printStackTrace();
      }
   }

   public void configure(Config config) throws BaseException {
      try {
         this.secret = config.getString("Encode.Secret");
         this.encoding = config.getString("Encode.Encoding");
         if (this.secret == null || "".equalsIgnoreCase(this.secret)) {
            this.secret = encodeSecretDefault;
         }

         if (this.encoding == null || "".equalsIgnoreCase(this.encoding)) {
            this.encoding = encodeEncodingDefault;
         }

         init();

      } catch (Exception e) {
         throw new BaseException(e.getMessage());
      }
   }

   public void configure(Properties properties) throws BaseException {
      try {
         this.secret = properties.getProperty("Encode.Secret");
         this.encoding = properties.getProperty("Encode.Encoding");
         // Create the key
         if (this.secret == null || "".equalsIgnoreCase(this.secret)) {
            this.secret = encodeSecretDefault;
         }

         if (this.encoding == null || "".equalsIgnoreCase(this.encoding)) {
            this.encoding = encodeEncodingDefault;
         }
         init();

      } catch (Exception e) {
         throw new BaseException(e.getMessage());
      }
   }

   /**
    * 
    */
   public Log getLogger() {
      return logger;
   }

   /**
    * 
    */
   public void setLogger(Log logger) {
      this.logger = logger;
   }

}
