package org.enhydra.dm.util;


import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.transaction.Status;
import javax.transaction.TransactionManager;


import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.data.CounterDO;
import org.enhydra.dm.data.CounterQuery;


/**
 * Numerator represents a <i>toolbox</i> for DODS based modules. There are static
 * methods only in charge of some specific task.
 * 
 */
public class Numerator {

   private static final String COUNTER_CACHE_PREFFIX = "Numerator.";

   private static final String COUNTER_CACHE_POSTFIX = ".CacheSize";
   
   private static Properties props;

   private static Map counterCachesSizes = new HashMap();

   private static Map counterCachesMax = new HashMap();

   private static Map counterCachesCurrent = new HashMap();

   private static Map counterCachesNext = new HashMap();

   private static final long DEFAULT_CACHE_SIZE = 100;

   private static int allocTryCount = 50;
   
   private static int cache_size = 100;
   
   private static String userTransactionJNDIPath = "java:comp/env/UserTransaction";



   // private constructor to prevent instantiate of Counter
   private Numerator() {
   }

   /**
    * Gets next value for counter named in objectName parameter if counter exists,
    * otherwise new one will be started with value 1.
    * 
    * @param objectName name of a counter
    * @return next value available for the counter
    * @exception Exception thrown if counter value cannot be acquired
    */
   public static synchronized BigDecimal getNext(String objectName) throws Exception {

      if (objectName == null) {
         throw new Exception("Object name parameter can't be null");
      }
      try {
         if (counterCachesNext.get(objectName) == null
             || counterCachesMax.get(objectName) == null
             || counterCachesNext.get(objectName)
                .equals(counterCachesMax.get(objectName))) {
            updateCaches(objectName);
         }
         counterCachesCurrent.put(objectName, counterCachesNext.get(objectName)); // next
         // free
         counterCachesNext.put(objectName,
                               ((BigDecimal) counterCachesNext.get(objectName)).add(new BigDecimal("1")));
         return (BigDecimal) counterCachesCurrent.get(objectName);
      } catch (Exception e) {
         System.err.println("Counter Id Allocator failed to allocate object id.");
         throw e;
      }
   }

   private static void updateCaches(String objectName) throws Exception {
      javax.transaction.Transaction mainOne = null;
      TransactionManager tm = null;
      // try it 50 times
      try {       
         tm = (TransactionManager) new InitialContext().lookup(userTransactionJNDIPath);       
         if (Status.STATUS_NO_TRANSACTION != tm.getStatus()) {
            mainOne = tm.suspend();
         }

      } catch (Exception e) {
         System.err.println("Can't allocate Id for object table " + objectName);
         throw e;
      }
      for (int iTry = 0; iTry < allocTryCount; iTry++) {
         try {
            tm.begin();
            CounterQuery qryCounter = new CounterQuery();
       
            qryCounter.requireUniqueInstance();
            qryCounter.setQueryName(objectName);
            CounterDO doCounter = qryCounter.getNextDO();

            BigDecimal dbNext;
            if (doCounter == null) {
               // insert new Counter
               doCounter = CounterDO.createVirgin();
               doCounter.setName(objectName);
               dbNext = new BigDecimal("1");
            } else {
               // increment existing Counter
               dbNext = doCounter.getThe_number();
            }
            BigDecimal dbLast = dbNext;
            dbNext = dbNext.add(BigDecimal.valueOf(getCacheSize(objectName)));

            doCounter.setThe_number(dbNext);
            doCounter.save();
            tm.commit();

            counterCachesNext.put(objectName, dbLast);
            counterCachesMax.put(objectName, dbNext);

            if (null != mainOne) {
               tm.resume(mainOne);
            }
            return;
         } catch (Exception e) {
            try {
               if (Status.STATUS_NO_TRANSACTION != tm.getStatus()) {
                  tm.rollback();
               }
            } catch (Exception _) {
            }
         }
      }
      // this line should normaly never be executed
      throw new Exception("Can't allocate Id for object table " + objectName);
   }
   

   public static String getUserTransactionJNDIPath() {
      return userTransactionJNDIPath;
   }

   public static void setUserTransactionJNDIPath(String userTransactionJNDIPath) {
      Numerator.userTransactionJNDIPath = userTransactionJNDIPath;
   }

   public static int getAllocTryCount() {
      return allocTryCount;
   }

   public static void setAllocTryCount(int allocTryCount) {
      Numerator.allocTryCount = allocTryCount;
   }

   public static int getCache_size() {
      return cache_size;
   }

   public static void setCache_size(int cache_size) {
      Numerator.cache_size = cache_size;
   }
   
   /**
    * Configure method that read parameters from Properties
    * 
    * @param properties
    * @throws BaseException
    */
   public static void configure(Properties properties) throws BaseException{
   
      Numerator.props=properties;
     
   };
   
   private static long getCacheSize(String objectName) {
      long cacheSize = -1;
      Object cs = counterCachesSizes.get(objectName);
      if (cs == null) {
         String propName = COUNTER_CACHE_PREFFIX + objectName + COUNTER_CACHE_POSTFIX;
         String cSize = Numerator.props.getProperty(propName);        
         if (cSize != null) {
            try {
               cacheSize = Long.parseLong(cSize);
               if (cacheSize <= 0) {
                  cacheSize = -1;
                  System.err.println("Wrong value for " + objectName + " cache size");
               }
            } catch (Exception ex) {
               cacheSize = -1;
               System.err.println("Wrong value for " + objectName + " cache size");
            }
         }
         if (cacheSize == -1) {
            
               cacheSize = DEFAULT_CACHE_SIZE;
           
         }
         counterCachesSizes.put(objectName, new Long(cacheSize));
      } else {
         cacheSize = ((Long) cs).longValue();
      }
      return cacheSize;
   }
  
}



