package org.enhydra.dm.util.filesys;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.api.util.filesys.FileChangeListenerImpl;

public class PropertyChangeListenerImpl extends FileChangeListenerImpl {
   private Properties props;

   public PropertyChangeListenerImpl(String fileName, long fileRefreshPeriod, Log logger) throws FileNotFoundException {
      super(fileName, fileRefreshPeriod, logger);
   }

   /**
    * Interface method implementation.
    * Load propertis file to input strim, make new <code>Properties</code> object
    * and reads a property list (key and element pairs) from the input stream.  
    * Method id called from constructor, and evey time properties file is changed. 
    * 
    * @param fileName name of changed file.
    */
   public void fileChanged(String fileName) {
      InputStream in = null;
      try {
         in = new FileInputStream(fileName);
         props = new Properties();
         props.load(in);
         if (logger != null) {
            logger.log(Log.INFORMATION, "Language peoperties file " + fileName + " is (re)loaded.");
         }
      } catch (IOException e) {
         if (logger != null) {
            logger.log(Log.WARNING, "Language peoperties file isn't found.");
         }
      } finally {
         try {
            if (in != null)
               in.close();
         } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
         }
      }
   }

   /**
    * @return <code>Properties</code> object
    */
   public Properties getProperties() {
      return props;
   }
}
