package org.enhydra.dm.util.filesys;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.api.util.filesys.FileChangeListenerImpl;

public class TemplateChangeListenerImpl extends FileChangeListenerImpl {
   private static TransformerFactory factory = null;

   private Templates templates;

   public TemplateChangeListenerImpl(String fileName, long fileRefreshPeriod, Log logger, TransformerFactory factory) throws FileNotFoundException {
      super(fileName, fileRefreshPeriod, logger);
      this.factory = factory;
   }

   /**
    * Interface method implementation.
    * Load xsl template file to input strim and make new <code>Template</code> object
    * using <code>TransformerFactory</code> and input stream.  
    * Method id called from constructor, and evey time template file is changed. 
    * 
    * @param fileName name of changed file.
    */
   public void fileChanged(String fileName) {
      InputStream in = null;
      try {
         in = new FileInputStream(fileName);
         if (factory == null) {
            factory = TransformerFactory.newInstance();
         }
         templates = factory.newTemplates(new StreamSource(in));
         if (logger != null) {
            logger.log(Log.INFORMATION, "Template file " + fileName + " is (re)loaded.");
         }
      } catch (IOException e) {
         if (logger != null) {
            logger.log(Log.ERROR, "Template file isn't found.");
         }
      } catch (TransformerConfigurationException e) {
         e.printStackTrace(); ///????
      } finally {
         try {
            if (in != null)
               in.close();
         } catch (IOException e) {
            e.printStackTrace();
         }
      }
   }

   /**
    * Metod uses <code>Template</code> object to get <code>Transformer</code> object.
    * 
    * @return <code>Transformer</code> object
    * @throws TransformerException
    */
   public Transformer getTransformer() throws TransformerException {
      Transformer transformer = null;
      transformer = templates.newTransformer();

      return transformer;
   }
}
