/*
 * MyTextPane                                                                              !:)==($)
 * <P>
 * @authors: Stefanovic Nenad, Puskas Vladimir, Bojanic Sasa
 * @Date:   July-November 2001.
 */

import javax.swing.*;
import java.util.*;
import java.beans.*;
import java.net.URL;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.datatransfer.*;
import javax.swing.text.html.*;
import java.awt.Color;

/**
 * Since JTextPane does not redefines cut, copy, and paste methods inherited from JTextComponent
 * there is a inconsistency in implementation. Although styled text can be drawn and typed in,
 * clipboard operations aren't working as they suppose to. Hence this class.<p>
 *
 * Applets are tipically run in sandboxed environments, so if one wants to use system clipboard
 * it is essential to configure one's machine properly.<br> Locate your JRE folder, and bellow it, 
 * there is <code>lib\security\java.security</code> file. There are lines similar to the next two:<p>
 *    <code>policy.url.1=file:${java.home}/lib/security/java.policy<br>
 *    <code>policy.url.2=file:${user.home}/.java.policy<p>
 * Edit eighter file and insert into appropriate <b>grant</b> section line as follows<p>
 *    <code>permission java.awt.AWTPermission "accessClipboard", "access";<p>
 * <b>Hint</b><br><code>.security</code> files you can edit using your favorite text editor or with
 * <code>${java.home}\bin\policytool.exe</code>
 */
public class MyTextPane extends JTextPane{

   static final private String signature = "!:)==($)";
/**
 * Constructs a new instance.
 *
 * There is not much to do. Let the superclass' constructor does the hard work.
 */
   public MyTextPane(){
      super();
      try {
         jbInit();
      }catch( Exception e ) {
         e.printStackTrace();
      }
      pasteDeletesSelection = true;
   }
/**
 * Transfers the currently selected range to the system clipboard, <b>removing</b>
 * the selection from the text. The current selection is reset. Does nothing for null selections.
 */
   public void cut(){
      copy( true );
   }

   public void copyExt(){
      super.copy();
   }
/**
 * Transfers the currently selected range to the system clipboard, <b>leaving</b> 
 * the selected text intact. The current selection is reset. Does nothing for null selections.
 */
   public void copy(){
      copy( false );
      setSelectionStart( getSelectionEnd());
   }
/**
 * Transfers the contents of the system clipboard into the associated text model. 
 * If there is no selection, the clipboard contents are inserted in front of the current insert 
 * position in the associated view. If the clipboard is empty, does nothing.
 * If there is a selection in the associated view, behavior is described in the following note.
 * <p><b>NOTE:</b><br> Paste operation depends on state which is set by setPasteDeletesSelection
 * method. When true selection is deleted prior to actual pasting, otherwise text is inserted in
 * front of selection. Default is true.
 * @see #setPasteDeletesSelection
 * @see #getPasteDeletesSelection
 */
   public void paste(){
      if( myClipboard == null )
         return;
      if( pasteDeletesSelection ){
         int selectionStart = getSelectionStart();
         int selectionEnd = getSelectionEnd();
         if( selectionStart < selectionEnd ){
            try{
               getStyledDocument().remove( selectionStart, selectionEnd - selectionStart );
            }catch( BadLocationException e ){
               e.printStackTrace();
            }
         }
      }
      try{
         String clip = ( String )( myClipboard.getContents( DataFlavor.stringFlavor ).getTransferData( DataFlavor.stringFlavor ));
// test on length is crucial because substring method will throw an exception for strigs shorter than signature
         if(( clip.length()> signature.length())&&( clip.substring( 0, signature.length()).equals( signature ))) {
           takeIn( clip );
         } else
            super.paste();
      }catch( Exception e ){
         e.printStackTrace();
      }
   }
/**
 * Long and unforgettable name of this method is chosen to explain itself.
 * @see #paste
 * @see #setPasteDeletesSelection
 * @return wether paste method would deleted selection.
 */   
   public boolean getPasteDeletesSelection(){
      return this.pasteDeletesSelection;
   }
/**
 * Long and unforgettable name of this method is chosen to explain itself.
 * @see #paste
 * @see #getPasteDeletesSelection
 * @param value specifying wether paste method would deleted selection
 */   
   public void setPasteDeletesSelection( boolean pasteDeletesSelection ){
      this.pasteDeletesSelection = pasteDeletesSelection;
   }
/* NOTE: Javadoc won't make this appear in generated documentation, 
 * still there are some private stuff, that need to be explained.
 *
 * One cannot know if system security settings will allow to use system clipboard, hence this
 * field. Working methods wouldn't know what clipboard they are accessing, they don't need to
 * know, while myClipboard is working properly.
 */
   private java.awt.datatransfer.Clipboard myClipboard;
/*
 * Because paste can act two ways this field bares what way it should act.
 */
   private boolean pasteDeletesSelection;
private boolean isInsideCut;
/*
 * Initializes the state of this instance.
 * This won't appear in javadoc generated files. Method itself is included,...
 * well I don't know why it is here.
 */
   private void jbInit() throws Exception{
      pickClipboardToUse();
   }
/*
 * CUT and COPY operations are the same except CUT deletes text it copied to clipboard. This
 * private method provides for both.
 */
   private void copy( boolean deleteToo ){
      int selectionStart = getSelectionStart();
      int selectionEnd = getSelectionEnd();
      if( selectionStart >= selectionEnd )
         return;
      StringSelection stringCopied = new StringSelection( returnCopy( selectionStart, selectionEnd ));
      myClipboard.setContents( stringCopied, stringCopied );
      if( deleteToo ){
    	  isInsideCut = deleteToo;
         try{
        	 ListAttribute selctedA = getListAttribute(getSelectionEnd());
        	 getStyledDocument().remove( selectionStart, selectionEnd - selectionStart );
        	 StyledDocument theDocument = getStyledDocument();
        	 Element element = theDocument.getParagraphElement(getSelectionStart());
        	 int stof = element.getStartOffset();
        	 int enof = element.getEndOffset();
        	 
        	 ListAttribute a = getListAttribute(stof);
        	 if (stof + 1 == enof) {
        		 a.setStatus(0);
        		 a.setStyle("none");
        		 a.setParagraphNo(0);
        		 a.setBullet(a.getStyle());
        	 } else {
        		 if (selctedA.getStatus() > 0) {    						
					try {
						selctedA.setStatus(0);
						selctedA.setStyle("none");
						theDocument.remove(getSelectionStart(), selctedA.getBulletLength());
						selctedA.setParagraphNo(0);
						selctedA.setBullet(a.getStyle());						
					} catch (BadLocationException e1) {								
						e1.printStackTrace();
					}
        		 }
        		 changeCharacterListAttribute(getSelectionStart(), enof, a);
        	 }
        	 manageNextAttribute(stof);
         }catch( BadLocationException e ){
            e.printStackTrace();
         } finally {
        	 isInsideCut = false;
         }
      }
   }
      
/*
 * Here it is. Method pickClipboardToUse tries to connect JTextPane to system clipboard.
 * If connection fails local clipboard is made, so cut, copy, paste functionality could
 * work as expected.
 */
   //private void pickClipboardToUse(){
   public void pickClipboardToUse(){
      if( myClipboard != null )
         return;
      try{
         System.getSecurityManager().checkSystemClipboardAccess();
         myClipboard = getToolkit().getSystemClipboard();
         System.out.println("Using system clipboard.");
      }catch( SecurityException se ){
         myClipboard = new Clipboard("TextEditor");
         System.out.println("Using local clipboard, because system isn't available.");
      }
   }
/*
 * Escape sequences used to store character and paragraph attributes in String are 
 * (all are preceded with backslash sign '\'):
 *
 * family   =   whatever font family is set its name is concatenated after this tag
 * size     =   size of font in decimal representation
 * style    =   (Bold, Italic, or Underlined)
 * align    =   number which return and accept static methods of StyleConstants class
 * color    =   rgb value
 *
 * private method takeIn does dirty work for paste method, parsing in content of myOwnClipboard 
 * and adding text into document together with attributes
 */
   private void takeIn(String myOwnClipboard){
	   boolean isEnd = false;
	   StyledDocument theDocument = getStyledDocument();
	   int insertHere = getSelectionStart();
	   AttributeSet cpas = getParagraphAttributes();
	   SimpleAttributeSet currentAS = new SimpleAttributeSet(cpas);
	            
	   Element element = theDocument.getParagraphElement(insertHere);
	   int stof = element.getStartOffset();
	   
	   //bullet attributes of the first paragraph witch is pasted
	   int start = myOwnClipboard.indexOf("#", signature.length()) + 1;
	   int end = start + 1;
	   int status = (new Integer(myOwnClipboard.substring(start, end)));
	   start = end + 1;
	   end = myOwnClipboard.indexOf("\\", start);
	   String style = myOwnClipboard.substring(start, end); 
	      
	   ListAttribute a = new ListAttribute(status, style);
	   currentAS.removeAttribute("list");
	   currentAS.addAttribute("list", a);
	      
	   //bullet attributes of the paragraph in witch we eill paste
	   ListAttribute oldA = (ListAttribute)element.getAttributes().getAttribute("list");
	   int oldStatus = 0;

	   //if paragraph in witch we will paste, is element of the list, save old status
	   //put 0 as status walue and remove bullet
	   if (oldA.getStatus() > 0) {
		   try {
			   oldStatus = oldA.getStatus();
			   oldA.setStatus(0);
			   theDocument.remove(stof, oldA.getBulletLength());
		   } catch (BadLocationException e) {
			   e.printStackTrace();
		   }
	   }
	   
	   //if we paste in the middle of paragraph, all characters in front of cursor
	   //must get the same attributeSet like those in the paharagraph witch will be pasted
	   insertHere = getSelectionStart();
	   changeCharacterListAttribute(stof, insertHere, a);
	   
	   //bullet attributes of the previous paragraph
	   ListAttribute pa = getListAttribute(stof - 1);
	   
	   for(int i = end + 1; i < myOwnClipboard.length(); ++i){
		   String currentChar = myOwnClipboard.substring(i, 1 + i);
		   insertHere = getSelectionStart();
		   if( currentChar.equals("\\")) {
			   if(( myOwnClipboard.charAt(i + 1)=='\\')){
				   try{
					   theDocument.insertString(insertHere, currentChar, currentAS);
					   i++;
					   continue;
				   }catch(BadLocationException e){
					   e.printStackTrace();
				   }
			   }
			   currentChar = myOwnClipboard.substring(i, myOwnClipboard.indexOf("\\", 1 + i));
			   switch(currentChar.charAt(2)){
			   		case 'u':
			   			//make new bullet attribute, and add it in new paragraph attributeSet
			   			status = (new Integer(currentChar.substring(8, 9)));
			   			style = currentChar.substring(10, currentChar.length());
			   			pa = a;
			   			a = new ListAttribute(status, style);
			   			currentAS = new SimpleAttributeSet(currentAS);
			   			currentAS.removeAttribute("list");
			   			currentAS.addAttribute("list", a);
			   			break;
			   		case 'l':
			   			try{
			   				if (a.getStatus() > 0) {
			   					if (pa != null && pa.getStyle().equals(a.getStyle())) {
			   						a.setParagraphNo(pa.getParagraphNo() + 1);
			   						a.setBullet(a.getStyle());
			   					} else {
			   						a.setParagraphNo(1);
			   						a.setBullet(a.getStyle());
			   					}
			   					theDocument.insertString(stof, a.getBullet(), currentAS);
			   				}
			   				insertHere = getSelectionStart();
			   				StyleConstants.setAlignment(currentAS,(new Integer(currentChar.substring(6,7))).intValue());
			   				theDocument.insertString(insertHere,"\n", currentAS);
			   				insertHere = getSelectionStart();
			   				
			   				moveCaretPosition(insertHere - 1);
			   				setParagraphAttributes(currentAS, false);
			   				moveCaretPosition(insertHere);
		    		  		
			   				element = theDocument.getParagraphElement(insertHere);
			   				stof = element.getStartOffset();
			   				isEnd = true;
			   			}catch( BadLocationException e ){
			   				e.printStackTrace();
			   			}
			   			break;
			   		case 't':
			   			if( currentChar.substring(6).equals("Bold"))
			   				StyleConstants.setBold(currentAS, true);
			   			else if( currentChar.substring(6).equals("Italic"))
			   				StyleConstants.setItalic(currentAS, true);
			   			else
			   				StyleConstants.setUnderline(currentAS, true);
			   			break;
			   		case 'a':
			   			StyleConstants.setFontFamily(currentAS, currentChar.substring(7));
			   			StyleConstants.setBold(currentAS, false);
			   			StyleConstants.setItalic(currentAS, false);
			   			StyleConstants.setUnderline(currentAS, false);
			   			break;
			   		case 'o':
			   			StyleConstants.setForeground(currentAS, new Color(new Integer(currentChar.substring(6)).intValue()));
			   			break;
			   		case 'i':   
			   			StyleConstants.setFontSize(currentAS,(new Integer(currentChar.substring(5))).intValue());
			   			break;
			   }
			   i += currentChar.length();
		   } else {
			   try{
				   theDocument.insertString(insertHere, currentChar, currentAS);
				   isEnd = false;
			   }catch(BadLocationException e){
				   e.printStackTrace();
			   }
		   }
	   }
	   //ako je poslednji znak koji je kopiran oznaka kraja reda
	   //a pasus u koji je pejstovano deo liste i pri tome 
	   //je pejstovano u sredinu tog pasusa, na pocetku mu je postavljen
	   //status na nulu i sklonjen bullet sada mu treba odrediti redni broj 
	   //na osnovu prethodnog pasusa, vratiti stari status i dodati bullet
	   if (isEnd) {
		   if (oldStatus != oldA.getStatus()) {
			   pa = getListAttribute(stof - 1);
			   if (pa.getStyle().equals(oldA.getStyle())) {
				   oldA.setParagraphNo(pa.getParagraphNo() + 1);
			   } else {
				   oldA.setParagraphNo(1);
			   }
			   oldA.setStatus(oldStatus);
			   oldA.setBullet(oldA.getStyle());
			   insertHere = getSelectionStart();
			   cpas = element.getAttributes();
			   try {
				   theDocument.insertString(stof, oldA.getBullet() ,cpas);
			   } catch (BadLocationException e) {
				   e.printStackTrace();
			   }
		   }
	   }
	   else {
		   insertHere = getSelectionStart();
		   if (insertHere != element.getEndOffset() - 1) {
			   changeCharacterListAttribute(insertHere, element.getEndOffset(), a);
		   }
		   
		   if (a.getStatus() > 0) {
			   if (pa != null && pa.getStyle().equals(a.getStyle())) {
				   a.setParagraphNo(pa.getParagraphNo() + 1);
				   a.setBullet(a.getStyle());
			   } else {
				   a.setParagraphNo(1);
				   a.setBullet(a.getStyle());
			   }
			   try {
				   theDocument.insertString(stof, a.getBullet(), currentAS);
			   } catch (BadLocationException e) {
				   e.printStackTrace();
			   }
		   }
		   setParagraphListAttribute(a);
	   }
	   manageNextAttribute(stof);
   }


   
 /** adding text into document together with attributes
  * @param fotext text for adding
  * @param overwrite indicates whether or not the previous text should be cleared before the new text are added.
 */ 
   public void loadTextf(String fotext,boolean overwrite){
	 if (overwrite){  
	   setText("");
	   }
	   takeIn(fotext);
   }
   
/*
 * private method returnCopy is heart'n'soul of both cut and copy methods. 
 * Running through selection character by character it makes String representation of styled 
 * text which would be stored in clipboard.
 */
   private String returnCopy(int selectionStart, int selectionEnd) {
	   boolean end = false;
	   StyledDocument theDocument = getStyledDocument();
	   AttributeSet characterAttributes = theDocument.getCharacterElement(selectionStart).getAttributes();
	   String result = signature;
	   String paragraphResult = shotOut(characterAttributes);
	   Element element = theDocument.getParagraphElement(selectionStart);
	   int stof = element.getStartOffset();
	   ListAttribute a = (ListAttribute)element.getAttributes().getAttribute("list");
	   int distinction = selectionStart - stof; 
	   if (distinction <= a.getBulletLength() && a.getStatus() > 0) {
		   selectionStart += (a.getBulletLength() - distinction);
	   }
	   
	   while(selectionStart < selectionEnd) {
		   AttributeSet currentAttributes = theDocument.getCharacterElement(selectionStart).getAttributes();
		   if(! characterAttributes.isEqual(currentAttributes)) {
			   paragraphResult += shotOut(currentAttributes);
			   characterAttributes = currentAttributes.copyAttributes();
		   }
		   try{
			   String theChar = theDocument.getText(selectionStart++, 1);
			   if(theChar.equals("\n")) {
				   a = (ListAttribute)theDocument.getParagraphElement(selectionStart-1).getAttributes().getAttribute("list");
				   result += "\\bullet#" + a.getStatus() + "#" + a.getStyle() + "\\";
				   result += paragraphResult;
				   result += "\\align"+ StyleConstants.getAlignment(theDocument.getParagraphElement(selectionStart).getAttributes()) + "\\";
				   paragraphResult = "";
				   if (a.getStatus() > 0) {
					   selectionStart += (a.getBulletLength());
				   }
				   end = true;
				}else{
				   paragraphResult += theChar;
				   if(theChar.equals("\\")) {
					   paragraphResult += theChar;
				   }
				   end = false;
			   }
		   }catch(BadLocationException e) {
			   e.printStackTrace();
		   }
	   } 
	   if (!end) {
		   a = (ListAttribute)theDocument.getParagraphElement(selectionStart-1).getAttributes().getAttribute("list");
		   result += "\\bullet#0#none\\";
		   result += paragraphResult;
	   }
	   return(result);
   }
  
/**
* Makes String representation of styled 
* text which would be stored in file.
*/   
    public String prepareTextForSave(){
	   String res="";
	   res=returnCopy(0,getText().length()-1);
	   return res;
   }
      
/*
 * private method shotOut returns String representation of current character attributes
 * (font family, size, and style)
 */
   private String shotOut( AttributeSet attributes ){
      String result = "\\family" + StyleConstants.getFontFamily( attributes )+"\\";
      result += "\\color" + StyleConstants.getForeground( attributes ).getRGB()+"\\";
      if( StyleConstants.isBold( attributes )){
         result += "\\styleBold\\";
      }
      if( StyleConstants.isItalic( attributes )){
         result += "\\styleItalic\\";
      }
      if( StyleConstants.isUnderline( attributes )){
         result += "\\styleUnderline\\";
      }
      return( result +"\\size"+ StyleConstants.getFontSize( attributes )+"\\" );
   }
   
   /**
    * Public mathod for getting paragraph's ListAttribute attribute.
    * @param dot - ofset that defines paragraph whose attribute this method returns
    * @return an atribute (instance of class ListAttribute) 
    */
   public ListAttribute getListAttribute (int dot){
	   ListAttribute a = null;
	   StyledDocument theDocument = getStyledDocument();
	   
	   if (dot > -1 && dot < theDocument.getLength()) {		   
		   Element prgel=theDocument.getParagraphElement(dot);
		   AttributeSet pas=prgel.getAttributes();
	   
		   if (pas.getAttribute("list") != null) {
			   a = (ListAttribute)pas.getAttribute("list");
		   }
	   }
	   return a;
   }
   
   /**
    * Public method that change paragraph's No and bullet(in order list)
    * in all paragraphs, from current to the lat paragraph in the list 
    * @param dot - current paragraph ofset
    */
   public void manageNextAttribute (int dot) {
	   StyledDocument theDocument = getStyledDocument();
	   
	   Element curPrgel = theDocument.getParagraphElement(dot);
	   int curStof = curPrgel.getStartOffset();
	   int curEnof = curPrgel.getEndOffset();
	   ListAttribute ca = getListAttribute(dot);
	   Element nextPrgel = null;
	   int nextStof = 0;
	   int nextEnof = 0;	   
	   ListAttribute na = null;
	   
	   if (curEnof + 1 < theDocument.getLength()) {		   
		   nextPrgel = theDocument.getParagraphElement(curEnof + 1);		   
		   nextStof = nextPrgel.getStartOffset();
		   nextEnof = nextPrgel.getEndOffset();
		   na = getListAttribute(curEnof + 1);
		   
		   //while it doesn't reach the end of the list (rather start of the next list), or
		   //paragraph that isn't in the list
		   while (na.getParagraphNo() > 1 && na.getStatus() > 0) {
				   				   
			   if (ca.getStyle().equals(na.getStyle())) {
				   na.setParagraphNo(ca.getParagraphNo() + 1);				   
			   } 
			   //this is executing after destroing bullet in the middle of the list
			   else {
				   na.setParagraphNo(1);
			   }
				   
			   if (na.getStatus() == 2) {
				   try {					   
					   na.setStatus(0);
					   theDocument.remove(nextStof, na.getBulletLength());
					   na.setStatus(2);
					   na.setBullet(na.getStyle());
					   theDocument.insertString(nextStof, na.getBullet(), nextPrgel.getAttributes());
				   } catch (BadLocationException e) {
					   e.printStackTrace();
				   }
			   }
				   
			   curPrgel = nextPrgel;
			   curStof = nextPrgel.getStartOffset();
			   curEnof = nextPrgel.getEndOffset();
			   ca = na;
				   
			   if (curEnof + 1 < theDocument.getLength()) {
				   nextPrgel = theDocument.getParagraphElement(curEnof + 1);
				   nextStof = nextPrgel.getStartOffset();
				   nextEnof = nextPrgel.getEndOffset();
				   na = getListAttribute(curEnof + 1);	
			   } else {
				   return;
			   }
		   }
	   }
   }
   
   /**
    * Public method that change paragraph's No and bullet(in order list)
    * in all paragraphs, from current to the lat paragraph in the list 
    * @param ca - current ListAttribute
    * @curPrgel - current paragraph
    */
   public void manageNextAttribute (ListAttribute ca, Element curPrgel) {
	   StyledDocument theDocument = getStyledDocument();
	   
	   int curStof = curPrgel.getStartOffset();
	   int curEnof = curPrgel.getEndOffset();

	   Element nextPrgel = null;
	   int nextStof = 0;
	   int nextEnof = 0;	   
	   ListAttribute na = null;
	   
	   if (curEnof + 1 < theDocument.getLength()) {		   
		   nextPrgel = theDocument.getParagraphElement(curEnof + 1);		   
		   nextStof = nextPrgel.getStartOffset();
		   nextEnof = nextPrgel.getEndOffset();
		   na = getListAttribute(curEnof + 1);
		   
		   //while it doesn't reach the end of the list (rather start of the next list), or
		   //paragraph that isn't in the list
		   while (na.getParagraphNo() > 1 && na.getStatus() > 0) {
				   				   
			   if (ca.getStyle().equals(na.getStyle())) {
				   na.setParagraphNo(ca.getParagraphNo() + 1);				   
			   } 
			   //this is executing after destroing bullet in the middle of the list
			   else {
				   na.setParagraphNo(1);
			   }
				   
			   if (na.getStatus() == 2) {
				   try {					   
					   na.setStatus(0);
					   theDocument.remove(nextStof, na.getBulletLength());
					   na.setStatus(2);
					   na.setBullet(na.getStyle());
					   theDocument.insertString(nextStof, na.getBullet(), nextPrgel.getAttributes());
				   } catch (BadLocationException e) {
					   e.printStackTrace();
				   }
			   }
				   
			   curPrgel = nextPrgel;
			   curStof = nextPrgel.getStartOffset();
			   curEnof = nextPrgel.getEndOffset();
			   ca = na;
				   
			   if (curEnof + 1 < theDocument.getLength()) {
				   nextPrgel = theDocument.getParagraphElement(curEnof + 1);
				   nextStof = nextPrgel.getStartOffset();
				   nextEnof = nextPrgel.getEndOffset();
				   na = getListAttribute(curEnof + 1);	
			   } else {
				   return;
			   }
		   }
	   }
   }
   
   public void changeCharacterListAttribute (int start, int end, ListAttribute a) {
	   StyledDocument theDocument = getStyledDocument();
	   AttributeSet cas = null;
	   SimpleAttributeSet characterAS = null; 
	   for (int i = start; i < end; i++) {
		   cas = getStyledDocument().getCharacterElement(i).getAttributes();
		   characterAS = new SimpleAttributeSet(cas);
		   characterAS.removeAttribute("list");
		   characterAS.addAttribute("list", a);
		   theDocument.setCharacterAttributes(i, 1, characterAS, true);
	   }
   }
   
   public void setParagraphListAttribute(ListAttribute a) {
	   SimpleAttributeSet emptyAS = new SimpleAttributeSet();
	   emptyAS.addAttribute("list", a);
	   setParagraphAttributes(emptyAS, false);
   }
   
   public boolean isCut() {
	   return isInsideCut;
   }
}
