/**
 * Servlet ServletEditor
 * <P>
 * @author: Stefanovic Nenad, Bojanic Sasa, Aleksandar Stojsavljevic
 * @Date:   August 2001 - May 2003 - May 2007
 */

package fops.presentation;

import java.io.File;
import java.io.StringReader;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.enhydra.util.fo.Fo2FopResponsePostProcessor;
import org.enhydra.xml.io.OutputOptions;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentation;
import com.lutris.appserver.server.httpPresentation.HttpPresentationComms;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;

public class ServletEditor implements HttpPresentation
{
	private Log log = null;
	
	private String DateHTML = null;
	
	private String NameHTML = null;
	
	private String SubjHTML = null;
	
	private String AppletHTML = null;
	
	private String xmlParam = null;
	
	private String xslParam = null;
	
	private String foParam = null;
	
	boolean noParams = false;
	
	public void run(HttpPresentationComms comms) throws Exception
	{
		if (log == null)
		{
			log = LogFactory.getLog(ServletEditor.class);
		}
		
		xmlParam = comms.request.getParameter("xml");
		xslParam = comms.request.getParameter("xsl");
		foParam = comms.request.getParameter("fo");
		DateHTML = comms.request.getParameter("date");
		NameHTML = comms.request.getParameter("name");
		SubjHTML = comms.request.getParameter("subject");
		AppletHTML = comms.request.getParameter("context");
		
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			if ((xmlParam != null) && (xslParam != null))
			{
				// Rendering a PDF from XML+XSL files
				System.out.println("PDF from XML: " + xmlParam + " and XSL: " + xslParam);
				File content = new File(xmlParam);
				Document page = builder.parse(content);
				OutputOptions opt = getPostProcessingOptionsForXml();
				comms.response.writeDOM(opt, page);
				
			}
			else if (foParam != null)
			{
				// Rendering a PDF from FO file
				File content = new File(foParam);
				Document page = builder.parse(content);
				OutputOptions opt = getPostProcessingOptions();
				comms.response.writeDOM(opt, page);
			}
			else if (DateHTML != null || NameHTML != null || SubjHTML != null || AppletHTML != null)
			{
				// Rendering a PDF from applet parameters
				StringReader content;
				content = inputText(DateHTML, NameHTML, SubjHTML, AppletHTML);
				Document page = builder.parse(new InputSource(content));
				OutputOptions opt = getPostProcessingOptions();
				comms.response.writeDOM(opt, page);
			}
			else
			{
				System.out.println("There is no any request parameter given!!!");
				StringReader content;
				
				content = defaultText();
				
				Document page = builder.parse(new InputSource(content));
				OutputOptions opt = getPostProcessingOptions();
				comms.response.writeDOM(opt, page);
			}
		}
		catch (NullPointerException e)
		{
			throw new ClientPageRedirectException("IndexPO.po");
		}
	}
	
	public OutputOptions getPostProcessingOptions() throws HttpPresentationException
	{
		OutputOptions ret = new OutputOptions();
		ret.setMIMEType("text/fo+fop-pdf");
		
		Map map = new HashMap();
		
		//TODO is this OK
		map.put(Fo2FopResponsePostProcessor.AUTHOR_PARAM_NAME, "fopApplet Demo");
		map.put(Fo2FopResponsePostProcessor.CREATION_DATE_PARAM_NAME, new Date());
		map.put(Fo2FopResponsePostProcessor.CREATOR_PARAM_NAME, "fopApplet");
		map.put(Fo2FopResponsePostProcessor.KEYWORDS_PARAM_NAME, "TAS TAF TDA fopApplet");
		map.put(Fo2FopResponsePostProcessor.PRODUCER_PARAM_NAME, "Together Application Framework");
		map.put(Fo2FopResponsePostProcessor.TARGET_RESOLUTION_PARAM_NAME, new Integer(300));
		map.put(Fo2FopResponsePostProcessor.TITLE_PARAM_NAME, "fopApplet Demo Document");
		ret.setFreeformOptions(map);
		return ret;
	}
	
	public OutputOptions getPostProcessingOptionsForXml() throws HttpPresentationException
	{
		OutputOptions ret = new OutputOptions();
		ret.setMIMEType("text/xml+fop-pdf");
		ret.addFreeformOption("Xml2FoTemplatePath", xslParam);
		return ret;
	}
	
	/**
	 * Method inputText Creates a XSLFO file with inserted parameters from HTML.
	 * Noncomptibility: FOP 0.19 - <fo:page-sequence master-name="simple"> FOP
	 * 0.20.4 - <fo:page-sequence master-reference="simple">
	 * 
	 * @return a StringReader
	 * 
	 */
	public StringReader inputText(String DateHTML, String NameHTML, String SubjHTML, String AppletHTML)
	{
		String text = new String();
		text += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
		text += "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n";
		text += "  <fo:layout-master-set>\n";
		text += "     <fo:simple-page-master master-name=\"simple\" page-height=\"29.7cm\" page-width=\"21cm\" "
				+ "margin-top=\"1cm\" margin-bottom=\"2cm\" margin-left=\"2.5cm\" margin-right=\"2.5cm\">\n";
		text += "        <fo:region-body margin-top=\"3cm\"/>\n";
		text += "        <fo:region-before extent=\"3cm\"/>\n";
		text += "        <fo:region-after extent=\"1.5cm\"/>\n";
		text += "     </fo:simple-page-master>\n";
		text += "  </fo:layout-master-set>\n";
		text += "  <fo:page-sequence master-reference=\"simple\">\n";
		text += "     <fo:flow flow-name=\"xsl-region-body\">\n";
		text += "        <fo:block font-size=\"18pt\" font-family=\"sans-serif\" line-height=\"24pt\" " + "space-after.optimum=\"15pt\" background-color=\"blue\" color=\"white\" "
				+ "text-align=\"center\" padding-top=\"3pt\">\n";
		text += "           Demo PDF file from applet \n";
		text += "        </fo:block>\n";
		text += "        <fo:table font-family=\"Helvetica\" space-after.optimum=\"20pt\">\n";
		text += "           <fo:table-column column-width=\"20mm\" />\n";
		text += "           <fo:table-column column-width=\"150mm\" />\n";
		text += "           <fo:table-body>\n";
		text += "              <fo:table-row>\n";
		text += "                 <fo:table-cell>\n";
		text += "                       <fo:block color=\"blue\" font-weight=\"bold\">Date:</fo:block>\n";
		text += "                 </fo:table-cell>\n";
		text += "                 <fo:table-cell>\n";
		text += "                    <fo:block>" + DateHTML + "</fo:block>\n";
		text += "                 </fo:table-cell>\n";
		text += "              </fo:table-row>\n";
		text += "              <fo:table-row>\n";
		text += "                 <fo:table-cell>\n";
		text += "                    <fo:block color=\"blue\" font-weight=\"bold\">Name:</fo:block>\n";
		text += "                 </fo:table-cell>\n";
		text += "                 <fo:table-cell>\n";
		text += "                    <fo:block>" + NameHTML + "</fo:block>\n";
		text += "                 </fo:table-cell>\n";
		text += "              </fo:table-row>\n";
		text += "              <fo:table-row>\n";
		text += "                 <fo:table-cell>\n";
		text += "                    <fo:block color=\"blue\" font-weight=\"bold\">Subject:</fo:block>\n";
		text += "                 </fo:table-cell>\n";
		text += "                 <fo:table-cell>\n";
		text += "                    <fo:block>" + SubjHTML + "</fo:block>\n";
		text += "                 </fo:table-cell>\n";
		text += "              </fo:table-row>\n";
		text += "           </fo:table-body>\n";
		text += "        </fo:table>\n";
		text += AppletHTML;
		text += "     </fo:flow>\n";
		text += "  </fo:page-sequence>\n";
		text += "</fo:root>\n";
		
		StringReader foText = new StringReader(text);
		
		return foText;
	}
	
	public StringReader defaultText()
	{
		String text = new String();
		text += "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
		text += "<fo:root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\">\n";
		text += "  <fo:layout-master-set>\n";
		text += "     <fo:simple-page-master master-name=\"simple\" page-height=\"29.7cm\" page-width=\"21cm\" "
				+ "margin-top=\"1cm\" margin-bottom=\"2cm\" margin-left=\"2.5cm\" margin-right=\"2.5cm\">\n";
		text += "        <fo:region-body margin-top=\"3cm\"/>\n";
		text += "        <fo:region-before extent=\"3cm\"/>\n";
		text += "        <fo:region-after extent=\"1.5cm\"/>\n";
		text += "     </fo:simple-page-master>\n";
		text += "  </fo:layout-master-set>\n";
		text += "  <fo:page-sequence master-reference=\"simple\">\n";
		text += "     <fo:flow flow-name=\"xsl-region-body\">\n";
		text += "        <fo:block font-size=\"18pt\" font-family=\"sans-serif\" line-height=\"24pt\" " + "space-after.optimum=\"15pt\" background-color=\"blue\" color=\"white\" "
				+ "text-align=\"center\" padding-top=\"3pt\">\n";
		text += "           Demo PDF file \n";
		text += "        </fo:block>\n";
		text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
		text += "           <fo:inline font-family=\"sans-serif\" font-size=\"18pt\" font-weight=\"bold\" >There is no any request parameter given!!!</fo:inline>\n";
		text += "        </fo:block>\n";
		text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
		text += "           <fo:inline font-family=\"sans-serif\" font-size=\"16pt\" ></fo:inline>\n";
		text += "        </fo:block>\n";
		text += "        <fo:block font-size=\"12pt\" font-family=\"sans-serif\" line-height=\"18pt\" space-after.optimum=\"3pt\" text-align=\"center\" white-space-collapse=\"false\" language=\"en_US\" hyphenate=\"true\">\n";
		text += "           <fo:inline font-family=\"sans-serif\" font-size=\"12pt\" >Please, see documentation for details.</fo:inline>\n";
		text += "        </fo:block>\n";
		text += "     </fo:flow>\n";
		text += "  </fo:page-sequence>\n";
		text += "</fo:root>\n";
		
		StringReader defaultText = new StringReader(text);
		
		return defaultText;
	}
	
}
