/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JtaDiscRack.java,v 1.2 2006/12/15 11:30:07 slobodan Exp $
 */

package jtaDiscRack;

import javax.naming.InitialContext;
import javax.transaction.UserTransaction;

import com.lutris.appserver.server.*;

import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.util.*;

import jtaDiscRack.spec.UserTransactionJNDI;

/**
 * The application object.
 * 
 * Application-wide data would go here.
 */
public class JtaDiscRack extends StandardApplication {
    
    /*
     *  A few methods you might want to add to.
     *  See StandardApplication for more details.
     */
    public void startup(Config appConfig) 
        throws ApplicationException {
        
        // Get the UserTransaction JNDI path from configuration (used for transaction lookup)
    	String db = null;
    	try {
			db = appConfig.getString("NameDatabaseManager/DefaultDatabase");
		} catch (ConfigException e1) {
			db = "sid1";
		}
    	String propertyName = "DatabaseManager.DB."+db+".XaUserTransactionLookupName";
        try{
        	jtaDiscRack.spec.UserTransactionJNDI.jNDIName = appConfig.getString(propertyName);
        }catch(ConfigException ex){
        	try{
        		propertyName = "DatabaseManager.DB."+db+".XATransactionManagerLookupName";
            	UserTransactionJNDI.jNDIName = appConfig.getString(propertyName,"java:comp/env/UserTransaction");
        	}catch(ConfigException exc){
        		UserTransactionJNDI.jNDIName = "java:comp/env/UserTransaction";
        	}
        }
        
        try {
            // Get UserTransaction (sync cache load)
        	UserTransaction ut = (UserTransaction)
               new InitialContext().lookup(UserTransactionJNDI.jNDIName);
        	// Set timeout for UserTransaction (sync cache load)
        	ut.setTransactionTimeout(60 * 30);
        	// Begin UserTransaction (enabling sync cache load)
        	ut.begin();
        	// StandardApplication startup (initalizes sync cache load)
            super.startup(appConfig);
            // No need for UserTransaction commit - only read (sync cache load)
            ut.rollback();
        } catch (Exception e) {
            throw new ApplicationException(e);
        }

    }
    
    public boolean requestPreprocessor(HttpPresentationComms comms)
        throws Exception{
        return super.requestPreprocessor(comms);
    }

    /**
     * This is an optional function, used only by the Multiserver's graphical
     * administration. This bit of HTML appears in the status page for this
     * application. You could add extra status info, for example
     * a list of currently logged in users.
     *
     * @return HTML that is displayed in the status page of the Multiserver.
     */
    public String toHtml() {
        return "This is <I>JtaDiscRack</I>";
    }
}

